<?php
	/**
	* - JINZORA | Web-based Media Streamer -  
	* 
	* Jinzora is a Web-based media streamer, primarily desgined to stream MP3s 
	* (but can be used for any media file that can stream from HTTP). 
	* Jinzora can be integrated into a CMS site, run as a standalone application, 
	* or integrated into any PHP website.  It is released under the GNU GPL.
	* 
	* - Resources -
	* - Jinzora Author: Ross Carlson <ross@jasbone.com>
	* - Web: http://www.jinzora.org
	* - Documentation: http://www.jinzora.org/docs	
	* - Support: http://www.jinzora.org/forum
	* - Downloads: http://www.jinzora.org/downloads
	* - License: GNU GPL <http://www.gnu.org/copyleft/gpl.html>
	* 
	* - Contributors -
	* Please see http://www.jinzora.org/modules.php?op=modload&name=jz_whois&file=index
	* 
	* - Code Purpose -
	* This is the media backend for the default XML cache adaptor.
	*
	* @since 05.10.04
	* @author Ross Carlson <ross@jinzora.org>
	*/
	
	class jzMediaNode extends jzRawMediaNode {
		function jzMediaNode($arg = array(),$mode="path") {
			$this->_constructor($arg,$mode);
		}
		
		
		/**
		 * Adds files to the cache based on ID3.
		 * 
		 * @author Ben Dodson
		 * @version 10/15/04
		 * @since 10/15/04
		 */
		function updateCache($recursive = true, $media_path = false, $showstatus = false, $force = false, $readID3 = true) {

			if ($recursive === false) {
				return false; // can't do it.
			}
			if ($media_path === false) {
				return false; // we also need a media path specified.
			}
			else {
				// it doesn't matter what root it was called from;
				// the full cache must be rebuilt.
				$root = &new jzMediaNode();
				$be = &new jzBackend();
				
				$root->updateCacheHelper("",$media_path,$be,$showstatus,$force,$installer);
			}
		}
		
		function updateCacheHelper($path,$media_path,&$be, $showstatus,$force = false,$readID3 = true) {
			global $audio_types, $video_types, $hierarchy,$default_art,$ext_graphic;
			
			$bestImage = "";
			$albums = array();

			if ($path != "")
				$dir = $media_path . "/" . $path;
			else
				$dir = $media_path;
			
			if (!is_readable($dir)) return false;
			if ($showstatus){
				//showStatus($media_path);
			}
			
			$updated = $be->getUpdated($dir);
			
			if (is_string($hierarchy))
				$hierarchy = explode("/",$hierarchy);
			
			if ($this->getLevel() > 0)
				die("Error: updateCacheHelper called from non-root.");
				
			if (!($handle = opendir($dir))) 
				die("Could not access directory $dir");
			
			while ($file = readdir($handle)) {
				if ($path == "")
					$fpath = $file;
				else
					$fpath = $path . "/" . $file;
				
				$mfpath = $media_path . "/" . $fpath;
				
				if ($file == "." || $file == "..") {
					continue;
				}
				else if (is_dir($mfpath)) {
					$this->updateCacheHelper($fpath,$media_path,$be,$showstatus,$force,$readID3);
				}
				else if (preg_match("/\.($ext_graphic)$/i", $file) && !stristr($file,".thumb.")) {
				  // An image
				  if (@preg_match("/($default_art)/i",$file)) {
				    $bestImage = $mfpath;
				  } else if ($bestImage == "") {
				    $bestImage = $mfpath;
				  }
				}
				  else if (preg_match("/\.($audio_types)$/i", $file) || preg_match("/\.($video_types)$/i", $file)) {
					if (date("U",filemtime($mfpath)) > $updated) {
						// Set path so when getFileName is called and the filepath was not found,
						// we get the correct path.
						$track = &new jzMediaTrack($mfpath);
						$track->playpath = $mfpath;
						$meta = $track->getMeta("file",$installer);
						
						// Now let's show status if we should
						if($showstatus){
							if (($_SESSION['jz_import_full_progress'] % 50 == 0) 
								or ($_SESSION['jz_import_full_progress'] == 0)
								or ($_SESSION['jz_import_full_progress'] == 1)){
								showStatus();
							}
						}

						// strip out weird characters.
						$genre = str_replace("/","-",$meta['genre']);
						$artist = str_replace("/","-",$meta['artist']);
						$album = str_replace("/","-",$meta['album']);
						$filename = str_replace("/","-",$meta['filename']);
						
						if ($genre == "" || $genre === false) {
						  $genre = "Unknown";
						}
						if ($artist == "" || $artist === false) {
						  $artist = "Unknown";
						}
						if ($album == "" || $album === false) {
						  $artist = "Unknown";
						}

						// TODO: 1) in inject, do a case-insensitive comparison.
						//       2) guess id3 fields based on filesystem...
						
						$arr = array();
						$norm = array($genre,$artist,$album,$filename);
						for ($i = 0; $i < sizeof($hierarchy); $i++) {
							switch ($hierarchy[$i]) {
								case "genre":
									$arr[] = $genre;
									break;
								case "artist":
									$arr[] = $artist;
									break;
								case "album":
									$arr[] = $album;
									break;
								case "track":
									$arr[] = $filename;
									break;	
								default:
									$arr[] = $norm[$i];
							}	
						}
						
						$_SESSION['jz_import_full_progress']++;
						
						if (($child = $this->inject($arr,$mfpath,'leaf')) !== false) {
							$child->playpath = $mfpath;
							$child->setMeta($meta,"cache");
							$album = $child->getAncestor("album");
							if ($album !== false) {
							  $albums[$album->getPath("String")] = true;
							}
						}
					}
				}
			}
			//$be->setUpdated($dir);
			if ($bestImage != "") {
			  foreach ($albums as $ap => $true) {
			    $album = new jzMediaNode($ap);
			    $album->addMainArt($bestImage);
			  }
			}
		}
        }

	class jzMediaTrack extends jzRawMediaTrack {
		function jzMediaTrack($arg = array(),$mode="path") {
			$this->_constructor($arg,$mode);
		}
	}
?>
