<?php
	/**
	* - JINZORA | Web-based Media Streamer -  
	* 
	* Jinzora is a Web-based media streamer, primarily desgined to stream MP3s 
	* (but can be used for any media file that can stream from HTTP). 
	* Jinzora can be integrated into a CMS site, run as a standalone application, 
	* or integrated intof any PHP website.  It is released under the GNU GPL.
	* 
	* - Resources -
	* - Jinzora Author: Ross Carlson <ross@jasbone.com>
	* - Web: http://www.jinzora.org
	* - Documentation: http://www.jinzora.org/docs	
	* - Support: http://www.jinzora.org/forum
	* - Downloads: http://www.jinzora.org/downloads
	* - License: GNU GPL <http://www.gnu.org/copyleft/gpl.html>
	* 
	* - Contributors -
	* Please see http://www.jinzora.org/modules.php?op=modload&name=jz_whois&file=index
	* 
	* - Code Purpose -
	* This is the media backend for the database adaptor.
	*
	* @since 05.10.04
	* @author Ben Dodson <bdodson@seas.upenn.edu>
	*/
	// Most classes should be included from header.php
	
	
	// The music root is $media_dir.
	class jzRawMediaNode extends jzMediaNodeClass {
		
		var $nodecount;
		var $leafcount;
		var $filepath;
		var $root;
		
		/**
		* Constructor wrapper for jzMediaNode.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/04
		* @since 5/13/04
		*/
		function jzRawMediaNode($par = array(), $mode = "path") {
			$this->_constructor($par,$mode);
		}
		
		/**
		* Constructor wrapper for jzMediaNode.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 11/12/04
		* @since 5/13/04
		*/
		function _constructor($par = array(),$mode) {
			global $media_dir;
			
			$this->nodecount = false;
			$this->leafcount = false;
			$this->filepath  = false;
			$this->root      = $media_dir;
			parent::_constructor($par,$mode);
		}

		function updateCache($recursive = true, $root = false, $showStatus = false, $force = false, $readTags = true) {
			global $sql_usr, $sql_type, $sql_pw, $sql_socket, $sql_db,$media_dir, $web_root, $root_dir;
			$be = &new jzBackend();

			// FAST CASE:
			// if we are non-recursive and haven't been modified, we are done.
			$my_dir = $root;
			if ($my_dir === false) $my_dir = $this->getFilePath();
			if ($my_dir == "/" || $my_dir == "" || !is_dir($my_dir)) {
			  if ($this->getLevel() == 0) {
				return false;
			  } else {
			    $parent = $this->getParent();
			    return $parent->updateCache(false,false,$showStatus,$force,$readTags);
			  }
			}
			
			
			if ($recursive === false) {
				$nodePath = $this->getPath("String");
				// If our naturalDepth is greater than 1, we need to update the cache
				// at lower levels.
				if ($this->getNaturalDepth() > 1) {
					if (!($handle = opendir($my_dir))) 
						die("Could not access directory $my_dir in database::updateCache");
				
					while ($file = readdir($handle)) {
						$nextPath = ($nodePath == "") ? $file : $nodePath . "/" . $file;
						if ($file == "." || $file == "..") {
							continue;
						} else if (is_dir($my_dir . "/" .$file)) {
							$next = &new jzMediaNode($nextPath);
							$next->setNaturalDepth($this->getNaturalDepth() - 1);
							$next->updateCache(false,$my_dir . "/" . $file,$showStatus,$force,$readTags);
						}
					}
				}
				// find out when we were last updated; if it was after our last modification,
				// we are done.
				$stamp = $be->getUpdated($my_dir);
				if ($stamp > date("U",filemtime($my_dir))) {
					return true;
				}
			}
			// Do we want this error check?
			//if ($my_dir === false || !is_dir($my_dir)) return;
			
			// connect to the database.
			if (!$link = jz_db_connect())
            	die ("could not connect to database.");
			
			if (!isset($nodePath)){$nodePath="";}
			$slashedNodePath = addSlashes($nodePath);
			$slashedFilePath = addSlashes($my_dir);
			$level = $this->getLevel();
            		// Mark my children as invalid if they exist in the DB.
			// Mark as valid only if/when they are found.
                     	$sql = "UPDATE jz_nodes SET valid = 'false' WHERE valid != 'perm'";
                     	$slash = ($level == 0) ? "" : "/";
                     	if ($recursive) {
	                     	$sql .= " AND level > $level";
	                }
	                else {
	                	$sql .= " AND level = ";
	                	$sql .= $level+1;
	                }
	                $sql .= " AND path LIKE '${slashedNodePath}${slash}%'";
	                $sql .= " AND filepath LIKE '${slashedFilePath}%'";

 			jz_db_query($link, $sql) || die (jz_db_error($link));
			
			$sql = "UPDATE jz_tracks SET valid = 'false' WHERE valid != 'perm'";
                     	$sql .= " AND path LIKE '${slashedNodePath}${slash}%'";
                     	$sql .= " AND filepath LIKE '${slashedFilePath}%'";
                     	if (!$recursive) {
                     		$sql .= " AND level = ";
                     		$sql .= $level+1;
                     	}
			jz_db_query($link, $sql) || die (jz_db_error($link));

			$this->updateCacheHelper($be, $link, $recursive, $my_dir, $showStatus, $force, $readTags);
			
			$be = &new jzBackend();
			$be->setUpdated();
			//jz_db_close($link);
		}

		/**
		 * Updates the cache using $this as the base node.
		 * 
		 * @author Ben Dodson <bdodson@seas.upenn.edu>
		 * @version 11/12/04
		 * @since 5/13/04
		 */
		function updateCacheHelper(&$be, &$link, $recursive, $root, $showStatus, $force = false, $readTags = true) {
		  global $sql_usr, $sql_type, $sql_pw, $sql_socket, $sql_db,
		    $audio_types, $video_types, $ext_graphic, $default_art,
		    $track_num_seperator, $hierarchy;


		  // The database adaptor builds itself based on the filesystem.
		  if ($root !== false)
		    $media_dir = $root;
		  else
		    $media_dir = $this->getFilePath();
		  
		  if (!is_readable($media_dir)) return false;
		  $mySlashedName = addSlashes($this->getName());
		  $pathArray = $this->getPath();
		  $nodePath = $this->getPath("String");
		  $slashedNodePath = addSlashes($nodePath); // SQL no likey the quotes.
		  $fullSlashedNodePath = addSlashes($media_dir);
		  
		  $level = $this->getLevel();              		
		  $ptype = findPType($this);
		  
		  /* Echo out our current status, if they want it: */
		  if ($showStatus){
		    showStatus($media_dir);
		  }
		  
		  // Now handle $this.
		  $mdate = filemtime($media_dir);
		  if ($mdate > ($curtime = time())) {
		    if (@touch($media_dir) === false) {
		      $mdate = $curtime - ($mdate - $curtime);
		    } else {
		      $mdate = filemtime($media_dir);
		    }
		  }
		  $stamp = $be->getUpdated($media_dir);		
		  
		  $sql = "INSERT INTO jz_nodes(name,path,filepath,ptype,level,date_added,my_id) ";
		  $sql .= "VALUES('$mySlashedName','$slashedNodePath','$fullSlashedNodePath','$ptype',$level,'$mdate','".uniqid("T")."')";			
		  // ADD MORE INFO TO DB.
		  if (!jz_db_query($link, $sql)) {
		    // the node was already in the database.
		    $sql = "UPDATE jz_nodes SET valid = 'true' ";
		    $sql .= "WHERE path LIKE '$slashedNodePath'";
		    jz_db_query($link, $sql);
		  }
		  writeLogData('importer',"Importing node: ". $mySlashedName. " - ". $fullSlashedNodePath);
		  
		  // Now move inside $this's path.
		  if (!($handle = opendir($media_dir))) 
		    die("Could not access directory $media_dir in database::updateCacheHelper.");
		  
		  // let's look for the best files to use while we are scanning this directory:
		  $leafcount = 0;
		  $nodecount = 0;
		  $bestImage = "";
		  $bestDescription = "";
		  
		  while ($file = readdir($handle)) {
		    $nextPath = ($nodePath == "") ? $file : $nodePath . "/" . $file;
		    $nextFile = $media_dir . "/" . $file;
		    $slashedNextFile = addSlashes($nextFile);
		    $slashedFileName = addSlashes($file);
		    $slashedFilePath = addSlashes($nextPath);
		    if ($file == "." || $file == "..") {
		      continue;
		    } else if (is_dir($nextFile)) {
		      $next = &new jzMediaNode($nextPath);
		      if ($recursive) {
				$next->updateCacheHelper($be, $link,true,$nextFile,$showStatus, $force, $readTags);
		      }
		      else {
			// NOT RECURSIVE, just mark as valid, or add if it wasn't there.
			$sdate = filemtime($nextFile);
			if ($sdate > ($curtime = time())) {
			  if (@touch($nextFile) === false) {
			    $sdate = $curtime - ($sdate - $curtime);
			  } else {
			    $sdate = filemtime($nextFile);
			  }
			}
			
			$spt = findPType($next);
			$sqln = "INSERT INTO jz_nodes(name,path,filepath,ptype,level,date_added,my_id)";
			$sqln .= " VALUES('$slashedFileName','$slashedFilePath','$slashedNextFile','$spt',$level+1,'$sdate','".uniqid("T")."')";
			
			$sqlu = "UPDATE jz_nodes SET valid = 'true' ";
			$sqlu .= "WHERE path LIKE '$slashedFilePath'";
			
			jz_db_query($link, $sqln) || jz_db_query($link,$sqlu);
		      }
		      $nodecount++;
		    } else {  // is a regular file.
		      // That means check for media file (leaf),
		      // or picture or text for $this.
		      if (preg_match("/\.(txt)$/i", $file)) {
			// TODO: GET THE CORRECT DESCRIPTION IN $bestDescription
			// $bestDescription = $slashedNodePath . "/" . addSlashes($file);
			// This will need testing...
			$descFile = $media_dir. "/". addSlashes($file);
			if (is_file($descFile)){
			  $bestDescription = addslashes(nl2br(implode("\n",file($descFile))));
			}
		      }
		      if (preg_match("/\.($ext_graphic)$/i", $file) && !stristr($file,".thumb.")) {
			$fav_image_name = $default_art;
				// An image
				if (@preg_match("/($fav_image_name)/i",$file)) {
			  		$bestImage = $slashedNextFile;
				} else if ($bestImage == "") {
			  		$bestImage = $slashedNextFile;
				}
		      } else if (preg_match("/\.($audio_types)$/i", $file) || preg_match("/\.($video_types)$/i", $file)) {
			// A media file
			// add it as an element.
			$leafcount++;

			$mdate = filemtime($nextFile);
			if ($mdate > ($curtime = time())) {
			  if (@touch($nextFile) === false) {
			    $mdate = $curtime - ($mdate - $curtime);
			  } else {
			    $mdate = filemtime($nextFile);
			  }
			}
			
			// First, try putting me in the DB.
			$slashedFileName = addSlashes($file);
			$slashedFilePath = ($slashedNodePath == "") ? $slashedFileName : $slashedNodePath . "/" . $slashedFileName;
			$fullSlashedFilePath = addslashes($media_dir) . "/" . $slashedFileName;
			$mid = uniqid("T");
			$sql = "INSERT INTO jz_nodes(name,path,filepath,ptype,level,date_added,leaf,my_id) ";
			$sql .= "VALUES('$slashedFileName','$slashedFilePath','$fullSlashedFilePath','track',$level+1,'$mdate','true','".$mid."') ";
			$updatesql = "UPDATE jz_nodes SET valid = 'true' WHERE path LIKE '$slashedFilePath'";
			jz_db_query($link,$sql) || jz_db_query($link,$updatesql);
			
			// Now, did they want to force this?
			if (($stamp < $mdate) or $force) {
			  $track = &new jzMediaTrack($nextPath);
			  $track->filepath = $track->getPath("String");
			  $track->playpath = $nextFile;
			  if ($readTags === true) {
			    $meta = $track->getMeta("file"); // read meta info from the file;
			  } else {
				$meta = array();
				$meta['bitrate'] = "";
				$meta['length'] = "";
				$meta['size'] = "";
				$meta['title'] = "";
				$meta['artist'] = "";
				$meta['album'] = "";
				$meta['year'] = "";
				$meta['number'] = "";
				$meta['genre'] = "";
				$meta['frequency'] = "";
				$meta['comment'] = "";
				$meta['lyrics'] = "";
				$meta['type'] = "";
			  }
			  
			  $pname = addSlashes($file);
			  $bitrate = $meta['bitrate'];
			  $length = $meta['length'];
			  $filesize = $meta['size'];
			  $name = addSlashes($meta['title']);
			  $artist = addSlashes($meta['artist']);
			  $album = addSlashes($meta['album']);
			  $year = addSlashes($meta['year']);
			  $track = $meta['number'];
			  $genre = addSlashes($meta['genre']);
			  $frequency = $meta['frequency'];
			  $description = addSlashes($meta['comment']);
			  $lyrics = addSlashes($meta['lyrics']);
			  $fileExt = $meta['type'];
			  
			  // Now let's see if there is a description file...
			  $desc_file = str_replace(".". $fileExt,"",$media_dir . "/" . $file). ".txt";
			  $long_description = "";
			  if (is_file($desc_file) and filesize($desc_file) <> 0){
			    // Ok, let's read the description file
			    $handle2 = fopen($desc_file, "rb");
			    $long_description = addslashes(fread($handle2, filesize($desc_file)));
			    fclose($handle2);
			  }
			  
			  
			  // Now let's see if there is a thumbnail for this track
			  $thumb_file = addslashes(searchThumbnail($media_dir . "/" . $file));
			  
			  $sql = "INSERT INTO jz_tracks(path,level,my_id,filepath,name,trackname,bitrate,filesize,frequency,length,lyrics,genre,artist,album,year,number,extension)
			       VALUES('$slashedFilePath',$level+1,'".$mid."','$fullSlashedFilePath','$pname','$name','$bitrate','$filesize','$frequency','$length','$lyrics','$genre','$artist','$album','$year','$track','$fileExt')";
			  
			  // Now let's update status and log this
			  $_SESSION['jz_import_full_progress']++;
			  writeLogData('importer',"Importing track: ". $fullSlashedFilePath);
			  
			  $updatesql = "UPDATE jz_tracks SET valid = 'true',
					level = $level+1,
					trackname = '$name',
					bitrate = '$bitrate',
					filesize = '$filesize',
					frequency = '$frequency',
					length = '$length',
					lyrics = '$lyrics',
					year = '$year',
					genre = '$genre',
					artist = '$artist',
					album = '$album',
					number = '$track',
					extension = '$fileExt'
					WHERE path LIKE '$slashedFilePath'";
	
			  jz_db_query($link,$sql) || jz_db_query($link,$updatesql);
			} else {
			  $sql = "UPDATE jz_tracks SET valid = 'true' WHERE path LIKE '$slashedFilePath'";
			  jz_db_query($link,$sql);
			}
			
			// last thing: add thumb and/or descriptions.
			$sql = "valid = 'true'";
			
			if (isset($thumb_file)) {
			  $sql .= ", main_art = '$thumb_file'";
			}
			if (isset($description)) {
			  $sql .= ", descr = '$description'";		
			}
			if (isset($long_description)) {
			  $sql = ", longdesc = '$long_description'";
			}
			
			jz_db_query($link,"UPDATE jz_nodes SET $sql WHERE path LIKE '$slashedFilePath'");
		      }
		    }
		  }
		  // Back to $this node:
		  // add new info to $this's database entry.
		  if ($readTags === true) {
		    $be->setUpdated($media_dir);
		  } else {
		    $be->setUpdated($media_dir,0); // pretend it hasn't been updated since 1979.
		  }
		  
		  
		  $sqlUpdate = "nodecount = $nodecount, leafcount = $leafcount";
		  if ($bestImage != "") {
		    $sqlUpdate .= ", main_art = '$bestImage'";
		  }
		  if ($bestDescription != "") {
		    //$bestDescription = addSlashes(file_get_contents($bestDescription));
		    $sqlUpdate .= ", longdesc = '$bestDescription'";
		  }
		  $sql = "UPDATE jz_nodes SET $sqlUpdate WHERE path LIKE '$slashedNodePath'";
		  jz_db_query($link,$sql) || die (jz_db_error($link));
		  
		  $slash = ($level == 0) ? "" : "/";
		  // all done; remove everything beyond $this's path that is still not valid.
		  $sql = "DELETE FROM jz_nodes WHERE ";
		  $sql .= "level > $level AND path LIKE '${slashedNodePath}${slash}%' AND valid = 'false'";
		  $sql .= " AND filepath LIKE '${fullSlashedNodePath}%'";
		  jz_db_query($link, $sql);
		  
		  $sql = "DELETE FROM jz_tracks WHERE ";
		  $sql .= "path LIKE '${slashedNodePath}${slash}%' AND valid = 'false'";
		  $sql .= " AND filepath LIKE '${fullSlashedNodePath}%'";
		  jz_db_query($link, $sql);
		}
		
		
		/**
		* Counts the number of subnodes $distance steps down.
		* $distance = -1 does a recursive count.
		*
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function getSubNodeCount($type='both', $distance=false) {
			global $sql_type, $sql_pw, $sql_socket, $sql_db, $sql_usr;
			
			if ($distance === false) {
				$distance = $this->getNaturalDepth();	
			}
			// alias:
			if ($type == "tracks") {
				$type = "leaves";
			}
			
			// can we do it quickly?
			if ($distance == 1 && !($this->nodecount === false)) {
			  if ($type == 'nodes') {
			    return $this->nodecount;
			  }
			  if ($type == 'leaves') {
			    return $this->leafcount;
			  }
			  return $this->nodecount + $this->leafcount;
			}
			if ($distance == 1) {
				if (!$link = jz_db_connect())
	                     		die ("could not connect to database.");
	                     		
				$pathArray = $this->getPath();
				$level = $this->getLevel();
				$pathString = addslashes($this->getPath("String"));
	                     	$sql = "SELECT nodecount,leafcount FROM jz_nodes WHERE path LIKE '${pathString}'";
	                     	
	                     	$results = jz_db_query($link,$sql);
	                     	jz_db_close($link);

	                     	if (!isset($results->data[0]))
	                     		return 0;
	                     		
	                     	if ($type == "nodes") {
	                     		return $results->data[0]['nodecount'];
	                     	}
	                     	if ($type == "leaves") {
	                     		return $results->data[0]['leafcount'];
	                     	}
	                     	else {
	                     		return $results->data[0]['leafcount'] + $results->data[0]['nodecount'];
	                     	}
			}
			
			$pathArray = $this->getPath();
			$level = $this->getLevel();
			$pathString = addslashes($this->getPath("String"));
			if ($pathString != "") { $pathString .= "/"; }
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     	
                     	// speed up the common case of counting all tracks.
                     	if ($distance == -1 && $type == "leaves" && $this->getLevel() == 0) {
                     		$sql = "SELECT COUNT(*) FROM jz_tracks";
	                     	$results = jz_db_query($link,$sql);
				jz_db_close($link);
				return $results->data[0][0];
                     	}
	
                     	if ($distance == -1) {
                     		$op = ">";
                     	}
                     	else {
                     		$op = "=";
                     		$level = $level + $distance;
                     	}
                     	
                     	$lim = "";
                     	if ($type == "leaves") {
                     		$lim = "AND leaf = 'true'";
                     	}
                     	else if ($type == "nodes") {
                     		$lim = "AND leaf = 'false'";
                     	}
                     	$sql = "SELECT COUNT(*) FROM jz_nodes WHERE level $op $level $lim AND path LIKE '${pathString}%'";
                     	$results = jz_db_query($link,$sql);
                     	jz_db_close($link);
			return $results->data[0][0];
		}



		/**
		* Returns the subnodes as an array.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 10/31/04
		* @since 5/14/2004
		*/
		function getSubNodes($type="nodes", $distance=false, $random=false, $limit=0, $hasArt = false) {
			global $sql_type, $sql_pw, $sql_socket, $sql_db, $sql_usr;
			
			if ($distance === false) {
				$distance = $this->getNaturalDepth();	
			}
			// alias:
			if ($type == "tracks") {
				$type = "leaves";
			}
			
			$pathArray = $this->getPath();
			$level = $this->getLevel();
			$pathString = addSlashes($this->getPath("String"));
			if ($pathString != "") { $pathString .= "/"; }
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");

          		if ($distance < 0) {
                     		$op = ">";
                     	}
                     	else {
                     		$op = "=";
                     		$level = $level + $distance;
                     	}      
                     	if ($type != "leaves" && $hasArt !== false) {
                     		$artString = "AND main_art != ''";
                     	} else {
                     		$artString = "";
                     	}
			
			
			// now the query.
			if ($type == "leaves") {
				$sql = "SELECT * FROM jz_tracks WHERE level $op $level AND hidden = 'false' AND path LIKE '${pathString}%'";
				
				if ($random) {
					$sql .= " ORDER BY rand()";
				}
				else {
					$sql .= " ORDER BY path,name";
					// should we ORDER BY number?
				}
				if ($limit > 0) {
					$sql .= " LIMIT $limit";
				}
				$results = jz_db_query($link,$sql);
				jz_db_close($link);
				$arr = array();
				for ($i = 0; $i < $results->rows; $i++) {
					$me = &new jzMediaTrack(stripSlashes($results->data[$i]['path']));
					// FILL META HERE. ***//
					if (isset($results->data[$i]['filepath']) && $results->data[$i]['filepath'] != "")
						$me->playpath = $results->data[$i]['filepath'];
					if ($results->data[$i]['trackname'] != "" && $results->data[$i]['trackname'] != "-")
						$me->title = stripSlashes($results->data[$i]['trackname']);
					$arr[] = $me;
				}
				return $arr;
			} else {
				$sql = "SELECT my_id,path,leaf,nodecount,leafcount,main_art,playcount FROM jz_nodes WHERE level $op $level
					AND hidden = 'false' AND path LIKE '${pathString}%' $artString";
				if ($type == "nodes") {
					$sql .= " AND leaf = 'false'";
				}
				if ($random) {
					$sql .= " ORDER BY rand()";
				}
				else {
					$sql .= " ORDER BY name,path";
				}
				if ($limit > 0) {
					$sql .= " LIMIT $limit";
				}
				$results = jz_db_query($link,$sql);
				jz_db_close($link);
				// have $results.
				$arr = array();
				for ($i = 0; $i < $results->rows; $i++) {
					if ($results->data[$i]['leaf'] == "false") {
						$me = &new jzMediaNode(stripSlashes($results->data[$i]['path']));
						$me->leafcount = $results->data[$i]['leafcount'];
						$me->nodecount = $results->data[$i]['nodecount'];
						$me->artpath = stripSlashes($results->data[$i]['main_art']);
						$me->myid = $results->data[$i]['my_id'];
						if ($me->artpath == "") {
						  $me->artpath = false;
						}
						$me->playcount = $results->data[$i]['playcount'];
						$arr[] = $me;
					}
					else {
						$arr[] = &new jzMediaTrack(stripSlashes($results->data[$i]['path']));
					}
				}
				return $arr;
			}
		}
	
		
		/**
		* Returns the hidden subnodes as an array.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 10/31/04
		* @since 10/31/04
		*/
		function getHiddenSubNodes($distance = false) {
			global $sql_type, $sql_pw, $sql_socket, $sql_db, $sql_usr;
		
			if ($distance === false) {
				$distance = $this->getNaturalDepth();	
			}
			
			$pathArray = $this->getPath();
			$level = $this->getLevel();
			$pathString = addSlashes($this->getPath("String"));
			if ($pathString != "") { $pathString .= "/"; }
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");

          		if ($distance < 0) {
                     		$op = ">";
                     	}
                     	else {
                     		$op = "=";
                     		$level = $level + $distance;
                     	}        
                     	
                     	$sql = "SELECT * FROM jz_nodes WHERE level $op $level AND hidden = 'true' AND path LIKE '${pathString}%'";
                     	
                     	$results = jz_db_query($link,$sql);
			jz_db_close($link);

			// have $results.
			$arr = array();
			for ($i = 0; $i < $results->rows; $i++) {
				if ($results->data[$i]['leaf'] == "false") {
					$me = &new jzMediaNode(stripSlashes($results->data[$i]['path']));
					$me->leafcount = $results->data[$i]['leafcount'];
					$me->nodecount = $results->data[$i]['nodecount'];
					$me->myid = $results->data[$i]['my_id'];
					$arr[] = $me;
				}
				else {
					$me = &new jzMediaTrack(stripSlashes($results->data[$i]['path']));
					$me->myid = $results->data[$i]['my_id'];
					$arr[] = $me;
				}
			}
			return $arr;
                     	      
		}
		
		/**
		* Alphabetical listing of a node.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/	
		function getAlphabetical($letter, $type = "nodes", $distance = false) {
			global $sql_type, $sql_pw, $sql_socket, $sql_db, $sql_usr,$compare_ignores_the;
			
                        $letter = strtolower($letter);
			if ($distance === false) {
				$distance = $this->getNaturalDepth();	
			}

			$pathString = addSlashes($this->getPath("String"));
			if ($pathString != "") { $pathString .= "/"; }
			
			$level = $this->getLevel();
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
			if ($distance == -1) {
			// recursive
				$op = ">";
			}
			else {
				$level = $level + $distance;
				$op = "=";
			}
			
			if ($type == "nodes") {
				$t = "AND leaf = 'false'";
			}
			

			if ($type == "tracks" || $type == "leaves") {
			  // Check the trackname:
			  if ($letter == "#") {
                            if ($compare_ignores_the != "false") {
                              $results = jz_db_query($link,"SELECT path FROM jz_tracks WHERE path LIKE '${pathString}%' AND (trackname REGEXP '^[0-9]' OR trackname REGEXP '^the [0-9]') ORDER BY trackname");
                            } else {
			      $results = jz_db_query($link,"SELECT path FROM jz_tracks WHERE path LIKE '${pathString}%' AND trackname REGEXP '^[0-9]' ORDER BY trackname");
                            }
			  } else if ($letter == "*") {
			    jz_db_query($link,"SELECT path FROM jz_tracks WHERE path LIKE '${pathString}%' ORDER BY trackname");
			  } else {
                            if ($compare_ignores_the != "false") {
                              if ($letter == "t") {
                                $results = jz_db_query($link,"SELECT path FROM jz_tracks WHERE path LIKE '${pathString}%' AND ((trackname LIKE '${letter}%' AND trackname NOT LIKE 'the %') OR (trackname LIKE 'the t%')) ORDER BY trackname");
                              } else {
                                $results = jz_db_query($link,"SELECT path FROM jz_tracks WHERE path LIKE '${pathString}%' AND (trackname LIKE '${letter}%' OR trackname LIKE 'the ${letter}%') ORDER BY trackname");
                              }
                            } else {
			      $results = jz_db_query($link,"SELECT path FROM jz_tracks WHERE path LIKE '${pathString}%' AND trackname LIKE '${letter}%' ORDER BY trackname");
                            }
			  }
			  
			}
			
			else if ($letter == "#") {
                          if ($compare_ignres_the != "false") {
				$results = jz_db_query($link,"SELECT path,leaf FROM jz_nodes 
				  WHERE level $op $level $t AND path LIKE '${pathString}%'
				  AND (name REGEXP '^[0-9]' OR name REGEXP '^the [0-9]') ORDER BY name");
                          } else {
				$results = jz_db_query($link,"SELECT path,leaf FROM jz_nodes 
				  WHERE level $op $level $t AND path LIKE '${pathString}%'
				  AND name REGEXP '^[0-9]' ORDER BY name");
				  //TODO: add special characters (IE, anything not a-zA-Z)
                          } 
			}
			else if ($letter == "*") {
				$results = jz_db_query($link,"SELECT path,leaf FROM jz_nodes 
				  WHERE level $op $level $t AND path LIKE '${pathString}%' ORDER BY name");
			}
			else {
                                if ($compare_ignores_the != "false") {
                                  if ($letter == "t") {
                                    $results = jz_db_query($link,"SELECT path,leaf FROM jz_nodes 
				      WHERE level $op $level $t AND path LIKE '${pathString}%'
				      AND ((name LIKE 't%' AND name NOT LIKE 'the %') OR (name LIKE 'the t%')) ORDER BY name");
                                  } else {
                                    $results = jz_db_query($link,"SELECT path,leaf FROM jz_nodes 
				      WHERE level $op $level $t AND path LIKE '${pathString}%'
				      AND (name LIKE '${letter}%' OR name LIKE 'the ${letter}%')  ORDER BY name");
                                  }
                                } else {
			 	  $results = jz_db_query($link,"SELECT path,leaf FROM jz_nodes 
				    WHERE level $op $level $t AND path LIKE '${pathString}%'
				    AND name LIKE '${letter}%' ORDER BY name");
                               }
			}
			jz_db_close($link);
			// have $results.
			$arr = array();
			for ($i = 0; $i < $results->rows; $i++) {
				if ($type != "tracks" && $type != "leaves" && $results->data[$i]['leaf'] == "false") {
					$arr[] = &new jzMediaNode(stripSlashes($results->data[$i]['path']));
				}
				else {
					$arr[] = &new jzMediaTrack(stripSlashes($results->data[$i]['path']));
				}
			}
			return $arr;
		}
	
	

		/**
		* Searches media
		* 
		* @author Ben Dodson
		* @version 12/26/04
		* @since 9/21/04
		*/
		function search($searchArray2, $type='both', $depth = -1, $limit=0, $op = "and", $metasearch = array(), $exclude = array()) {
		  global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
		
		  // alias:
		  if ($type == "tracks") {
		    $type = "leaves";
		  }
		  
		  if ($depth === false) {
		    $depth = $this->getNaturalDepth();
		  }
		  
			// allow strings as well as arrays for searching:
			if (is_string($searchArray2) && $op != "exact") {	
				if (stristr($searchArray2,"\"") === false) {
					if ($searchArray2 == "") {
						$searchArray = array();
					} else {
						$searchArray = explode(" ",$searchArray2);
					}
				} else { // gets nasty..
					$open_quote = false;
					$searchArray = array();
					$word = "";
					for ($i = 0; $i < strlen($searchArray2); $i++) {
						if ($searchArray2[$i] == ' ' && $open_quote == false) {
							$searchArray[] = $word;
							$word = "";
						} else if ($searchArray2[$i] == '"') {
							$open_quote = !$open_quote;
						} else {
							$word .= $searchArray2[$i];
						}
					}
					if ($word != "") {
						$searchArray[] = addSlashes($word);
					}
				}
			} else {
				$searchArray = $searchArray2;
			}
			
			// exclude array, too:
			if (is_string($exclude)) {
				if ($exclude == "") {
					$excludeArray = array();
				} else if (stristr($exclude,"\"") === false) {
					if ($exclude == "") {
						$excludeArray = array();
					} else {
						$excludeArray = explode(" ",$exclude);
					}
				} else { // gets nasty..
					$open_quote = false;
					$excludeArray = array();
					$word = "";
					for ($i = 0; $i < strlen($exclude); $i++) {
						if ($exclude[$i] == ' ' && $open_quote == false) {
							$excludeArray[] = $word;
							$word = "";
						} else if ($exclude[$i] == '"') {
							$open_quote = !$open_quote;
						} else {
							$word .= addSlashes($exclude[$i]);
						}
					}
					if ($word != "") {
						$excludeArray[] = $word;
					}
				}
			} else {
				$excludeArray = $exclude;
			}
			
			
			// Now that we have search array, let's addSlashes here so we don't have to later.
			for ($i = 0; $i < sizeof($searchArray); $i++) {
				$searchArray[$i] = addSlashes($searchArray[$i]);
			}
			
			// Clean up:
			$tmp = array();
			if (is_array($searchArray)) {
				foreach ($searchArray as $term) {
					if (!($term == "" || $term == " ")) {
						$tmp[] = $term; 
					}
				}
				$searchArray = $tmp;
			}
			
			$tmp = array();
			foreach ($excludeArray as $term) {
				if (!($term == "" || $term == " ")) {
					$tmp[] = $term; 
				}
			}
			$excludeArray = $tmp;


			// SEARCH:
			$constraints = array();
		

			// LYRICS: a different kind of search.
			if ($type == "lyrics") {
				if ($this->getLevel() > 0) {
					$constraints[] = "path LIKE '".addSlashes($this->getPath("String"))."/%'";
				}
			
				// Level
				if ($depth < 0) {
					$lvl = $this->getLevel();
					$constraints[] = "level > $lvl";
				} else {
					$lvl = $this->getLevel() + $depth;
					$constraints[] = "level = $lvl";
				}
			
				if ($op == "exact") {
					$constraints[] = "lyrics LIKE '%$searchArray2'%";
				} else {
					if ($op == "or") {
						$OPR = "OR";
					} else { // and
						$OPR = "AND";
					}
					$string = "";
					if (sizeof($searchArray) > 0) {
						$string .= "lyrics LIKE '%$searchArray[0]%'";
					}
					for ($i = 1; $i < sizeof($searchArray); $i++) {
						$string .= "$OPR lyrics LIKE '%$searchArray[$i]%'";
					}
					$constraints[] = $string;
				}
		     
				// TERMS TO EXCLUDE
				if ($excludeArray != array()) {
					$string = "(lyrics NOT LIKE";
					$string .= " '%$excludeArray[0]%'";
					for ($i = 1; $i < sizeof($excludeArray); $i++) {
						$string .= " AND lyrics NOT LIKE '%$excludeArray[$i]%'";	
					}
					$string .= ")";
					$constraints[] = $string;
				}


				if ($constraints == array()) {
					die("Error: no constraints in search.");
				}
				$sql = "SELECT path FROM jz_tracks WHERE";
				$sql .= " ($constraints[0])";
				for ($i = 1; $i < sizeof($constraints); $i++) {
					$sql .= " AND ($constraints[$i])";
				}
				if ($limit > 0) {
					$sql .= " LIMIT $limit";
				}
		    
				if (!$link = jz_db_connect())
					die ("could not connect to database.");
		    
				$results = jz_db_query($link, $sql);
				jz_db_close($link);
				$return = array();
				
				if ($results === false) {
					return $return;
				}
				
				foreach ($results->data as $row) {
					$return[] = &new jzMediaTrack(stripSlashes($row['path']));
				}
				
				return $return;
			}  


			// MEDIA SEARCH (not lyrics)
			if ($this->getLevel() > 0) {
				$constraints[] = "path LIKE '".addSlashes($this->getPath("String"))."/%'";
			}
			
			// Type
			if ($type == "leaves") {
				$constraints[] = "leaf = 'true'";
			}	else if ($type == "nodes") {
				$constraints[] = "leaf = 'false'";
			}
			
			// Level
			if ($depth < 0) {
				$lvl = $this->getLevel();
				$constraints[] = "level > $lvl";
			} else {
				$lvl = $this->getLevel() + $depth;
				$constraints[] = "level = $lvl";
			}
		  
			// String
			if ($op == "exact") {
				$searchArray2 = str_replace("\"","",$searchArray2);
				$constraints[] = "name LIKE '$searchArray2'";
			} else if ($op == "or") { // "or"
				if (sizeof($searchArray) > 0) {
					$string = "name LIKE";
					$string .= " '%$searchArray[0]%'";
					for ($i = 1; $i < sizeof($searchArray); $i++) {
						$string .= " OR name LIKE '%$searchArray[$i]%'";
					}
				}
				$constraints[] = $string;
			} else { // "and"
				// first match at least part in our name:
				if (sizeof($searchArray) > 0 ) {
					$string = "((name LIKE";
					// if at a specific level, don't worry about full path:
					if ($lvl > 0) {
						$string .= " '%$searchArray[0]%')";
						// Otherwise, let's try to avoid repeat results:
					} else {
						$string .= " '%$searchArray[0]%' AND path NOT LIKE '%$searchArray[0]%/%')";
					}
					for ($i = 1; $i < sizeof($searchArray); $i++) {
						$string .= " OR (name LIKE '%$searchArray[$i]%' AND path NOT LIKE '%$searchArray[$i]%/%')";
					}
					$string .= ")";
				
					// Now require the rest to be in the path.
					$string .= " AND (path LIKE";
					$string .= " '%$searchArray[0]%'";
					for ($i = 1; $i < sizeof($searchArray); $i++) {
						$string .= " AND path LIKE '%$searchArray[$i]%'";
					}
					$string .= ")";
					$constraints[] = $string;
				}
			}
			
			// Stuff to exclude:
			if ($excludeArray != array()) {
				$string = "(path NOT LIKE";
				$string .= " '%$excludeArray[0]%'";
				for ($i = 1; $i < sizeof($excludeArray); $i++) {
					$string .= " AND path NOT LIKE '%$excludeArray[$i]%'";	
				}
				$string .= ")";
				$constraints[] = $string;
			}
			
			// Put it all together.
			if ($constraints == array()) {
				die("Error: no constraints in search.");
			}
			
			$sql = "SELECT path,leaf FROM jz_nodes WHERE";
			$sql .= " ($constraints[0])";
			for ($i = 1; $i < sizeof($constraints); $i++) {
				$sql .= " AND ($constraints[$i])";
			}
			if ($limit > 0) {
				$sql .= " LIMIT $limit";
			}
			if (!$link = jz_db_connect())
				die ("could not connect to database.");
				
			$results = jz_db_query($link, $sql);
			jz_db_close($link);
			$return = array();
		  
			if ($results === false) {
				return $return;
			}
			
			for ($i = 0; $i < $results->rows; $i++) {
				if ($results->data[$i]['leaf'] == "true") {
					$return[] = &new jzMediaTrack(stripSlashes($results->data[$i]['path']));
				} else {
					$return[] = &new jzMediaNode(stripSlashes($results->data[$i]['path']));
				}
			}
			
			if ($type == "leaves" && $metasearch != array()) {
				$return = filterSearchResults($return,$metasearch);
			}
			
			return $return;
		}
		


		/**
		* Add this node to the featured list.
		* 
		* @author Ben Dodson
		* @version 6/8/04
		* @since 6/8/04
		*/
		function addFeatured() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET featured = 'true' WHERE path = '$path'");
		}
		
		/**
		* Removes this node from the featured list.
		* 
		* @author Ben Dodson
		* @version 6/8/04
		* @since 6/8/04
		*/
		function removeFeatured() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET featured = 'false' WHERE path = '$path'");
		}

		/**
		* Removes this node from the featured list.
		* 
		* @author Ben Dodson
		* @version 6/8/04
		* @since 6/8/04
		*/
		function isFeatured() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT featured FROM jz_nodes WHERE path = '$path'");
                     	if ($results->data[0]['featured'] == "true") { return true; }
                     	else { return false; }
		}
		
		/**
		* Removes this node from the featured list.
		* 
		* @author Ben Dodson
		* @version 6/8/04
		* @since 6/8/04
		*/
		function getFeatured($distance = -1, $limit = 1) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if ($distance === false) {
				$distance = $this->getNaturalDepth();	
			}
			
			$slash = ($this->getLevel() == 0) ? "" : "/";
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
			if ($path != "") { $path .= "/"; }
			
			if ($limit > 0) {
				$lim = " LIMIT $limit";
			} else {
				$lim = "";
			}
			
			
			if ($distance <= 0) {
				$dis = "";
			}
			else {
				$level = $this->getLevel();
				$level += $distance;
				$dis = "AND level = '$level'";
			}
			
                     	$results = jz_db_query($link, "SELECT path FROM jz_nodes WHERE path LIKE '${path}%' AND featured = 'true' $dis ORDER BY rand() $lim");

			$vals = array();
			for ($i = 0; $i < $results->rows; $i++) {
				$vals[] = &new jzMediaNode(stripSlashes($results->data[$i][path]));	
			}
			return $vals;
		}
		
		/**
		* Returns the 'ptype' of this node.
		* 
		* @author Ben Dodson
		* @version 10/31/04
		* @since 10/31/04
		*/
		function getPType() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if ($this->getLevel() == 0) return "root";				
			if (!$link = jz_db_connect())
                		die ("could not connect to database.");
                		
                	$path = addSlashes($this->getPath("String"));
                	$results = jz_db_query($link, "SELECT ptype FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                	return stripslashes($results->data[0]['ptype']);
		}
		
		
		/**
		* Sets the 'ptype' of this node.
		* 
		* @author Ben Dodson
		* @version 10/31/04
		* @since 10/31/04
		*/
		function setPType($type) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
								
			if (!$link = jz_db_connect())
                		die ("could not connect to database.");
                	$ptype = addSlashes($ptype);
                	
                	$path = addSlashes($this->getPath("String"));
                	jz_db_query($link, "UPDATE jz_nodes SET ptype='$type' WHERE path = '$path'");
		}
		
		/**
		* Adds a request.
		* 
		* @author Ben Dodson
		* @version 9/2/04
		* @since 9/2/04
		*/
		function addRequest($entry, $comment, $user, $type = "request") {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			$entry = addSlashes($entry);
			$comment = addSlashes($comment);
			$user = addSlashes($user);
			$type = addSlashes(strtolower($type)); // ...you never know :)
			$path = addSlashes($this->getPath("String"));
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
			
			jz_db_query($link, "INSERT INTO jz_requests(entry,comment,user,type,path)
                     	                  VALUES('$entry','$comment','$user','$type','$path')") || die(jz_db_error($link));
		}
		
		
		/**
		* Gets the requests
		* 
		* @author Ben Dodson
		* @version 9/2/04
		* @since 9/2/04
		*/
		function getRequests($distance = -1, $type = "all") {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			$path = addSlashes($this->getPath("String"));
			
			$type = strtolower($type);
			if ($type == "all") {
				$typequery = "";
			} else {
				$typequery = "AND type LIKE '$type'";
			}
			
			if ($this->getLevel() == 0) {
			  $slash = '';
			} else {
			  $slash = '/';
			}
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
			
			$results = jz_db_query($link, "SELECT * FROM jz_requests WHERE path LIKE '${path}${slash}%' $typequery");
			
			$return = array();
			foreach ($results->data as $a) {
				$el = array();
				$el['id'] = $a['my_id'];
				$el['entry'] = stripSlashes($a['entry']);
				$el['comment'] = stripSlashes($a['comment']);
				$el['user'] = stripSlashes($a['user']);
				$el['type'] = stripSlashes($a['type']);
				$el['path'] = stripSlashes($a['path']);
				
				if ($distance <= 0) {
					$return[] = $el;
				} else {
					$temp = &new jzMediaNode($el['path']);
					if ($temp->getLevel() - $this->getLevel() == $distance) {
						$return[] = $el;
					}
				}
			}
			return $return;
		}
		
		
		/**
		* Removes request of given $id.
		* 
		* @author Ben Dodson
		* @version 9/2/04
		* @since 9/2/04
		*/
		function removeRequest($id) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	jz_db_query($link, "DELETE FROM jz_requests WHERE my_id=$id") || die(jz_db_error($link));
		}
	
	
		/**
		* Adds link to the given node.
		* 
		* 
		* @author Ben Dodson
		* @version 9/18/04
		* @since 9/18/04
		*/
		function addLink ($node) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			$parent = addSlashes($this->getPath("String"));
			$type = ($node->isLeaf()) ? 'leaf' : 'node';
			$path = addSlashes($node->getPath("String"));
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
			
			// let's remove it so we don't have duplicates.
			$this->removeLink($node);
			
			// now we'll add it.
			jz_db_query($link, "INSERT INTO jz_links(parent,path,type)
                     	                  VALUES('$parent','$path','$type')") || die(jz_db_error($link));
			
		}
		
		/**
		* Removes the link to the given node.
		* 
		* 
		* @author Ben Dodson
		* @version 9/18/04
		* @since 9/18/04
		*/
		function removeLink ($node) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			$parent = addSlashes($this->getPath("String"));
			$type = ($node->isLeaf()) ? 'leaf' : 'node';
			$path = addSlashes($node->getPath("String"));
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
			
			jz_db_query($link, "DELETE FROM jz_links WHERE parent = '$parent' AND path = '$path' AND type = '$type'") || die(jz_db_error($link));
		}
		
		
		/**
		* Gets the node's links.
		* 
		* 
		* @author Ben Dodson
		* @version 9/18/04
		* @since 9/18/04
		*/
		function getLinks($type = 'both') {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			// alias:
			if ($type == "tracks") {
				$type = "leaves";
			}
			
			$parent = addSlashes($this->getPath("String"));
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
			
			if ($type == 'both') {
				$tstring = "";
			}
			if ($type == 'nodes') {
				$tstring = "AND type = 'node'";
			}
			if ($type == 'leaves') {
				$tstring = "AND type = 'leaf'";
			}
			
			$results = jz_db_query($link, "SELECT * FROM jz_links WHERE parent = '$parent' $tstring");

			$return = array();
			foreach ($results->data as $a) {
				if ($a['type'] == 'leaf') {
					$return[] = &new jzMediaTrack(stripSlashes($a['path']));
				}
				else {
					$return[] = &new jzMediaNode(stripSlashes($a['path']));
				}
			}
			return $return;
		}
			
		/**
		 * Injects a leaf or a node into $this.
		 * If sizeof($path) > 1, does so 'recursively'
		 * 
		 * @author Ben Dodson
		 * @version 10/15/04
		 * @since 10/15/04
		 */	
		function inject($path, $filepath, $type = "leaf") {
		  global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db,$media_dir,$hierarchy;

		  if (is_string($path)) {
		    // todo: be more flexible (be carefully of '://')
		    return false;
		  }

		  if (!$link = jz_db_connect())
		    die ("could not connect to database.");
		  
		  if ($type == "track") {
		    $type = "leaf";
		  }
		  // Handle $path[0].
		  if ($path == array()) {
		    return $this;
		  }
		  else {
		    $head = array_shift($path);
		    $nextpath = $this->getPath();
		    $nextpath[] = $head;
		    $nexttrack = &new jzMediaTrack($nextpath);
		    $nextnode = &new jzMediaNode($nextpath);
		    
		    $ptype = findPType($this);
		    
		    $media_path = "." . $media_dir;
		    if (file_exists($filepath)) {
		      $date = filemtime($filepath);
		    } else {
		      $date = 0;
		    }				
		    
		    if (sizeof($path) == 0) {
		      // just add $next to cache.
		      $tlevel = $this->getLevel()+1;
		      $tpath = addSlashes(implode("/",$nextpath));
		      $tfilepath = addSlashes($filepath);
		      $thead = addSlashes($head);
		      
		      $tptype = addSlashes($hier[$this->getLevel()]);
		      if ($tptype == "track") {
				$tptype = "disk";
		      }
		      
		      if ($type == "leaf") {
				$mid = uniqid("T");
				$sql = "INSERT INTO jz_nodes(path,ptype,level,name,leaf,date_added,filepath,valid,my_id)";
				$sql .= " VALUES('$tpath','track',$tlevel,'$thead','true','$date','$tfilepath','perm','".$mid."')";
				jz_db_query($link,$sql);

				$sql = "INSERT INTO jz_tracks(path,level,name,filepath,valid,my_id) VALUES('$tpath','$tlevel','$thead','$tfilepath','perm','".$mid."')";
				if (jz_db_query($link,$sql)) {	
			 	 $ppath = addSlashes($this->getPath("String"));
			  	$sql = "UPDATE jz_nodes SET leafcount = leafcount+1 WHERE path = '$ppath'";
			  	jz_db_query($link,$sql);
			 	 return $nexttrack;
				}
				else {
			 	 return false;
				}
			
		      }
		      else {
			// Remember, INSERT will fail if the path already exists.
			// this is ok.
			$sql = "INSERT INTO jz_nodes(path,ptype,level,name,leaf,date_added,valid,my_id)";
			$sql .= " VALUES('$tpath','$tptype',$tlevel,'$thead','false','$date','perm','".uniqid("T")."')";
			if (jz_db_query($link,$sql)) {
			  $ppath = addSlashes($this->getPath("String"));
			  $sql = "UPDATE jz_nodes SET nodecount = nodecount+1 WHERE path = '$ppath'";
			  jz_db_query($link,$sql);	
			}
			
			return $nextnode;
		      }
		    }
		    else {
		      $npath = addSlashes($nextnode->getPath("String"));
		      $nlevel = $nextnode->getLevel();
		      $nhead = addSlashes($head);
		      
		      $nptype = findPType($nextnode);
		      
		      $sql = "INSERT INTO jz_nodes(path,ptype,level,name,leaf,date_added,valid,my_id)";
		      $sql .= " VALUES('$npath','$nptype',$nlevel,'$nhead','false','$date','perm','".uniqid("T")."')";
		      if (jz_db_query($link,$sql)) {
			$ppath = addSlashes($this->getPath("String"));
			if ($this->getLevel() == 0) {
			  $sql = "UPDATE jz_nodes SET nodecount = nodecount+1 WHERE level = 0";
			} else {
			  $sql = "UPDATE jz_nodes SET nodecount = nodecount+1 WHERE path = '$ppath'";
			}
			
			jz_db_query($link,$sql);	
		      }
		      return $nextnode->inject($path,$filepath,$type);
		    }
		  }
		}		


		/**
		*
		* @author Ross Carlson
		* @since 1/27/05
		* @version 1/27/05
		*
		* @param $type (defaults to nodes)
		* @param $distance (distance from where we are, defaults no false)
		* @param $limit how many items to return (defaults to 10)
		**/
		function getMostDownloaded($type = "nodes", $distance = false, $limit = 10) {
			if ($distance === false) {
				$distance = $this->getNaturalDepth();	
			}
			
			$pathArray = $this->getPath();
			$level = $this->getLevel();
			$pathString = addSlashes($this->getPath("String"));
			if ($pathString != "") { $pathString .= "/"; }
			
			$sql = "SELECT path,leaf FROM jz_nodes";
			
			if (!$link = jz_db_connect())
				die ("could not connect to database.");
			
			if ($distance <= 0) {
				$sql .= " WHERE level > $level";
			} else {
				$level = $level + $distance;
				$sql .= " WHERE level = $level";
			}
			
			$sql .= " AND path LIKE '${pathString}%' and dlcount <> 0";
			
			if ($type == "tracks" || $type == "leaves") {
				$sql .= " AND leaf = 'true'";
			} else if ($type == "nodes") {
				$sql .= " AND leaf = 'false'";
			}
			
			$sql .= " ORDER BY dlcount desc LIMIT $limit";

			$results = jz_db_query($link,$sql);
			$retArray = array();
			for ($i = 0; $i < $results->rows; $i++) {
				if ($results->data[$i]['leaf'] == "false") {
					$retArray[] = new jzMediaNode(stripSlashes($results->data[$i]['path']));
				} else {
					$retArray[] = new jzMediaTrack(stripSlashes($results->data[$i]['path']));
				}
			}
		
			// Now let's close out
			jz_db_close($link);
			
			// Now let's return the data
			return $retArray;
		}

		/**
		*
		* @author Ross Carlson
		* @since 1/27/05
		* @version 1/27/05
		*
		* @param $type (defaults to nodes)
		* @param $distance (distance from where we are, defaults no false)
		* @param $limit how many items to return (defaults to 10)
		**/
		function getMostPlayed($type = "nodes", $distance = false, $limit = 10) {
			if ($distance === false) {
				$distance = $this->getNaturalDepth();	
			}
			
			$pathArray = $this->getPath();
			$level = $this->getLevel();
			$pathString = addSlashes($this->getPath("String"));
			if ($pathString != "") { $pathString .= "/"; }
			
			$sql = "SELECT path,leaf,playcount FROM jz_nodes";
			
			if (!$link = jz_db_connect())
				die ("could not connect to database.");
			
			if ($distance <= 0) {
				$sql .= " WHERE level > $level";
			} else {
				$level = $level + $distance;
				$sql .= " WHERE level = $level";
			}
			
			$sql .= " AND path LIKE '${pathString}%' and playcount <> 0";
			
			if ($type == "tracks" || $type == "leaves") {
				$sql .= " AND leaf = 'true'";
			} else if ($type == "nodes") {
				$sql .= " AND leaf = 'false'";
			}
			
			$sql .= " ORDER BY playcount desc LIMIT $limit";

			$results = jz_db_query($link,$sql);
			$retArray = array();
			for ($i = 0; $i < $results->rows; $i++) {
				if ($results->data[$i]['leaf'] == "false") {
				  $a = new jzMediaNode(stripSlashes($results->data[$i]['path']));
				  $a->playcount = $results->data[$i]['playcount'];
				  $retArray[] = $a;
				} else {
					$retArray[] = new jzMediaTrack(stripSlashes($results->data[$i]['path']));
				}
			}
		
			// Now let's close out
			jz_db_close($link);
			
			// Now let's return the data
			return $retArray;
		}


		/**
		*
		* @author Ben Dodson
		* @since 1/27/05
		* @version 1/27/05
		*
		* @param $type (defaults to nodes)
		* @param $distance (distance from where we are, defaults no false)
		* @param $limit how many items to return (defaults to 10)
		**/
		function getMostViewed($type = "nodes", $distance = false, $limit = 10) {
			if ($distance === false) {
				$distance = $this->getNaturalDepth();	
			}
			
			$pathArray = $this->getPath();
			$level = $this->getLevel();
			$pathString = addSlashes($this->getPath("String"));
			if ($pathString != "") { $pathString .= "/"; }
			
			$sql = "SELECT path,leaf,playcount,viewcount FROM jz_nodes";
			
			if (!$link = jz_db_connect())
				die ("could not connect to database.");
			
			if ($distance <= 0) {
				$sql .= " WHERE level > $level";
			} else {
				$level = $level + $distance;
				$sql .= " WHERE level = $level";
			}
			
			$sql .= " AND path LIKE '${pathString}%' and viewcount <> 0";
			
			if ($type == "tracks" || $type == "leaves") {
				$sql .= " AND leaf = 'true'";
			} else if ($type == "nodes") {
				$sql .= " AND leaf = 'false'";
			}
			
			$sql .= " ORDER BY viewcount desc LIMIT $limit";

			$results = jz_db_query($link,$sql);
			$retArray = array();
			for ($i = 0; $i < $results->rows; $i++) {
			  if ($results->data[$i]['leaf'] == "false") {
			    $a = new jzMediaNode(stripSlashes($results->data[$i]['path']));
			  } else {
			    $a = new jzMediaTrack(stripSlashes($results->data[$i]['path']));
			  }
				$a->playcount = $results->data[$i]['playcount'];
				$a->viewcount = $results->data[$i]['viewcount'];
				$retArray[] = $a;
			}
		
			// Now let's close out
			jz_db_close($link);
			
			// Now let's return the data
			return $retArray;
		}

		
		/**
		*
		* @author Ben Dodson
		* @since 1/27/05
		* @version 1/27/05
		* @param $type (defaults to nodes)
		* @param $distance (distance from where we are, defaults no false)
		* @param $limit how many items to return (defaults to 10)
		*
		**/
		function getRecentlyAdded($type = "nodes", $distance = false, $limit = 10) {
		  if ($distance === false) {
		    $distance = $this->getNaturalDepth();	
		  }
		  
		  $pathArray = $this->getPath();
		  $level = $this->getLevel();
		  $pathString = addSlashes($this->getPath("String"));
		  if ($pathString != "") { $pathString .= "/"; }
		  
		  $sql = "SELECT path,leaf FROM jz_nodes";

		  if (!$link = jz_db_connect())
		    die ("could not connect to database.");
		  
		  if ($distance <= 0) {
		    $sql .= " WHERE level > $level";
		  }
		  else {
		    $level = $level + $distance;
		    $sql .= " WHERE level = $level";
		  }

		  $sql .= " AND path LIKE '${pathString}%'";
		  
		  if ($type == "tracks" || $type == "leaves") {
		    $sql .= " AND leaf = 'true'";
		  } else if ($type == "nodes") {
		    $sql .= " AND leaf = 'false'";
		  }
		  $sql .= " ORDER BY date_added desc LIMIT $limit";
		  $results = jz_db_query($link,$sql);
		  $retArray = array();
		  for ($i = 0; $i < $results->rows; $i++) {
		    if ($results->data[$i]['leaf'] == "false") {
		      $retArray[] = new jzMediaNode(stripSlashes($results->data[$i]['path']));
		    } else {
		      $retArray[] = new jzMediaTrack(stripSlashes($results->data[$i]['path']));
		    }
		  }
		  
		  // Now let's close out
		  jz_db_close($link);
		  
		  // Now let's return the data
		  return $retArray;
		}

		/**
		* Gets the top rated elements.
		*
		* @author Ben Dodson
		* @since 1/27/05
		* @version 1/27/05
		* @param $type (defaults to nodes)
		* @param $distance (distance from where we are, defaults no false)
		* @param $limit how many items to return (defaults to 10)
		*
		**/
		function getTopRated($type = "nodes", $distance = false, $limit = 10) {
		  if ($distance === false) {
		    $distance = $this->getNaturalDepth();	
		  }
		  
		  $pathArray = $this->getPath();
		  $level = $this->getLevel();
		  $pathString = addSlashes($this->getPath("String"));
		  if ($pathString != "") { $pathString .= "/"; }
		  
		  $sql = "SELECT path,leaf FROM jz_nodes";

		  if (!$link = jz_db_connect())
		    die ("could not connect to database.");
		  
		  if ($distance <= 0) {
		    $sql .= " WHERE level > $level";
		  }
		  else {
		    $level = $level + $distance;
		    $sql .= " WHERE level = $level";
		  }

		  $sql .= " AND path LIKE '${pathString}%' AND rating_val != 0";
		  
		  if ($type == "tracks" || $type == "leaves") {
		    $sql .= " AND leaf = 'true'";
		  } else if ($type == "nodes") {
		    $sql .= " AND leaf = 'false'";
		  }
		  $sql .= " ORDER BY rating_val desc,rating_count desc LIMIT $limit";
		  $results = jz_db_query($link,$sql);
		  $retArray = array();
		  for ($i = 0; $i < $results->rows; $i++) {
		    if ($results->data[$i]['leaf'] == "false") {
		      $retArray[] = new jzMediaNode(stripSlashes($results->data[$i]['path']));
		    } else {
		      $retArray[] = new jzMediaTrack(stripSlashes($results->data[$i]['path']));
		    }
		  }
		  
		  // Now let's close out
		  jz_db_close($link);
		  
		  // Now let's return the data
		  return $retArray;
		}

		/**
		*
		* @author Ben Dodson
		* @since 1/27/05
		* @version 1/27/05
		* @param $type (defaults to nodes)
		* @param $distance (distance from where we are, defaults no false)
		* @param $limit how many items to return (defaults to 10)
		*
		**/
		function getRecentlyPlayed($type = "nodes", $distance = false, $limit = 10) {
		  if ($distance === false) {
		    $distance = $this->getNaturalDepth();	
		  }
		  
		  $pathArray = $this->getPath();
		  $level = $this->getLevel();
		  $pathString = addSlashes($this->getPath("String"));
		  if ($pathString != "") { $pathString .= "/"; }
		  
		  $sql = "SELECT path,leaf FROM jz_nodes";

		  if (!$link = jz_db_connect())
		    die ("could not connect to database.");
		  
		  if ($distance <= 0) {
		    $sql .= " WHERE level > $level";
		  }
		  else {
		    $level = $level + $distance;
		    $sql .= " WHERE level = $level";
		  }

		  $sql .= " AND path LIKE '${pathString}%' AND lastplayed != 0";
		  
		  if ($type == "tracks" || $type == "leaves") {
		    $sql .= " AND leaf = 'true'";
		  } else if ($type == "nodes") {
		    $sql .= " AND leaf = 'false'";
		  }
		  $sql .= " ORDER BY lastplayed desc LIMIT $limit";
		  $results = jz_db_query($link,$sql);
		  $retArray = array();
		  for ($i = 0; $i < $results->rows; $i++) {
		    if ($results->data[$i]['leaf'] == "false") {
		      $retArray[] = new jzMediaNode(stripSlashes($results->data[$i]['path']));
		    } else {
		      $retArray[] = new jzMediaTrack(stripSlashes($results->data[$i]['path']));
		    }
		  }
		  
		  // Now let's close out
		  jz_db_close($link);
		  
		  // Now let's return the data
		  return $retArray;
		}


		function generateStats() {
		  $stats = array();
		  if ($this->getLevel() == 0) {
		    $pathstring = "";
		    $fullpathstring = "";
		  } else {
		    $pathstring = "path LIKE '" . addSlashes($this->getPath("String")) . "/%' AND ";
		    $fullpathstring = "WHERE path LIKE '" . addSlashes($this->getPath("String")) . "/%'";
		  }

		  $link = jz_db_connect();
		  
		  $results = jz_db_query($link, "SELECT sum(filesize) FROM jz_tracks WHERE $pathstring filesize != '-' AND filesize > 0");
		  $stats['total_size'] = $results->data[0][0];
		  
		  $results = jz_db_query($link, "SELECT sum(length) FROM jz_tracks WHERE $pathstring length != '-' AND length > 0");
		  $stats['total_length'] = $results->data[0][0];
		  
		  $results = jz_db_query($link, "SELECT count(*) FROM jz_tracks $fullpathstring");
		  $stats['total_tracks'] = $tracks = $results->data[0][0];
		  
		  $results = jz_db_query($link, "SELECT avg(bitrate) FROM jz_tracks WHERE $pathstring bitrate != '-' AND bitrate > 0");
		  $stats['avg_bitrate'] = round($results->data[0][0],2);
		  
		  $results = jz_db_query($link, "SELECT avg(length) FROM jz_tracks WHERE $pathstring length != '-' AND length > 0");
		  $stats['avg_length'] = round($results->data[0][0],0);


		  $results = jz_db_query($link, "SELECT avg(filesize) FROM jz_tracks WHERE $pathstring filesize != '-' AND filesize > 0");
		  $stats['avg_size'] = round($results->data[0][0],2);
		  
		  $results = jz_db_query($link, "SELECT avg(year) FROM jz_tracks WHERE $pathstring year != '-' AND year > 1000");
		  $stats['avg_year'] = round($results->data[0][0],2);
		  
		  $str = "";
		  // stringize stuff:
		  $stats['avg_length_str'] = stringize_time($stats['avg_length']);
		  $stats['total_length_str'] = stringize_time($stats['total_length']);
		  $stats['total_size_str'] = stringize_size($stats['total_size']);
		  
		  if (($d = distanceTo('genre',$this)) !== false)
		    $stats['total_genres'] = $this->getSubNodeCount('nodes',$d);
		  else
		    $stats['total_genres'] = 0;
		  
		  if (($d = distanceTo('artist',$this)) !== false)
		    $stats['total_artists'] = $this->getSubNodeCount('nodes',$d);
		  else
		    $stats['total_artists'] = 0;
		  
		  if (($d = distanceTo('album',$this)) !== false)
		    $stats['total_albums'] = $this->getSubNodeCount('nodes',$d);
		  else
		    $stats['total_albums'] = 0;
		  
		  // nodes at track level are disks.
		  if (($d = distanceTo('track',$this)) !== false)
		    $stats['total_disks'] = $this->getSubNodeCount('nodes',$d);
		  else
		    $stats['total_disks'] = 0;
		  
		  $this->stats = $stats;
		}
		


		/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
		 * Do NOT modify the below: modify overrides.php instead,        *
		 * change to jinzora/backend, and run `php global_include.php`   *
		 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
		// begin global_include: overrides.php
		/* * * * * * * * * * * * * * * * * * *
		 *            Overrides              *
		 * * * * * * * * * * * * * * * * * * */
		
		/**
		* Returns the date the node was added.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function getDateAdded() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT date_added FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	return $results->data[0]['date_added'];
		}

/**
 * Checks whether or not
 * this media has been updated within $days days.
 * If not, returns false. Otherwise, returns
 * the number of days since the most recent media was added.
 *
 * @author Ben Dodson
 * @version 4/9/05
 * @since 4/9/05
 **/
function newSince($days) {
  $time = $curtime = time();
  $time -= ($days*24*60*60);

  if ($this->getLevel() == 0) {
    $pathstring = "%";
  } else {
    $pathstring = $this->getPath("String");
    $pathstring .= "/%";
  }
  $sql = "SELECT date_added FROM jz_nodes WHERE ";
  $sql .= "path LIKE '$pathstring' AND date_added >= $time ORDER BY date_added desc LIMIT 1";


  if (!$link = jz_db_connect())
    die ("could not connect to database.");
  
  $results = jz_db_query($link, $sql);
  jz_db_close($link);

  if (sizeof($results->data) == 0) {
    return false;
  } else {
    return ceil(abs($curtime - $results->data[0]['date_added']) / (24*60*60));
  }
}


/**
 * Returns the element's ID
 *
 * @author Ben Dodson
 * @version 3/11/05
 * @since 3/11/05
 **/
function getID() {
  if (isset($this->myid) && $this->myid !== false) {
    return $this->myid;
  } else {
    if (!$link = jz_db_connect())
      die ("could not connect to database.");
    
    $path = addSlashes($this->getPath("String"));
    $results = jz_db_query($link, "SELECT my_id FROM jz_nodes WHERE path = '$path'");
    jz_db_close($link);
    return $results->data[0]['my_id'];
  }
}

/**
 * Converts an id to a path
 *
 * @author Ben Dodson
 * @version 3/11/05
 * @since 3/11/05
 **/
function idToPath($id) {
    if (!$link = jz_db_connect())
      die ("could not connect to database.");
    
    $results = jz_db_query($link, "SELECT path FROM jz_nodes WHERE my_id = '$id'");
    jz_db_close($link);
    return $results->data[0]['path'];
}

		/**
		* Returns the number of times the node has been played.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function getPlayCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT playcount FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	return $results->data[0]['playcount'];
		}
		
		
		/**
		* Increments the node's playcount, as well
		* as the playcount of its parents.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function increasePlayCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
			$sql = "UPDATE jz_nodes SET playcount = playcount+1, lastplayed = " . time();

                     	jz_db_query($link, "$sql  WHERE path = '$path'");
			                     	
                     	if (sizeof($ar = $this->getPath()) > 0) {
                     		array_pop($ar);
                     		$next = &new jzMediaNode($ar);
				$next->increasePlayCount();
                     	}
			jz_db_close($link);
		}
	
		/**
		* Sets the elements playcount.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function setPlayCount($n) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET playcount = $n WHERE path = '$path'");
			jz_db_close($link);
		}


	
               /**
		* Increments the node's view count
		*
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 3/15/2005
		* @since 3/15/2005
		*/
		function increaseViewCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET viewcount = viewcount+1 WHERE path = '$path'");
                     	
			jz_db_close($link);
		}


		/**
		* Returns the number of times the node has been viewed.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 3/15/2005
		* @since 3/15/2005
		*/
		function getViewCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT viewcount FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	return $results->data[0]['viewcount'];
		}


		/**
		* Returns the number of times the node has been downloaded.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function getDownloadCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT dlcount FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	return $results->data[0]['dlcount'];
		}
		
		
		/**
		* Increments the node's download count, as well
		* as the count of its parents.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function increaseDownloadCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET dlcount = dlcount+1 WHERE path = '$path'");
                     	
                     	if (sizeof($ar = $this->getPath()) > 0) {
                     		array_pop($ar);
                     		$next = &new jzMediaNode($ar);
				$next->increasePlayCount();
                     	}
			jz_db_close($link);
		}

		/**
		* Sets the elements download count.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function setDownloadCount($n) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET dlcount = $n WHERE path = '$path'");
			jz_db_close($link);
		}

		/**
		* Returns the main art for the node.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/04
		* @since 5/14/04
		*/
		function getMainArt($dimensions = false, $createBlank = true) {
		  global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db,$jzSERVICES;
		  
		  if (!$link = jz_db_connect())
		    die ("could not connect to database.");
		  
		  $path = addSlashes($this->getPath("String"));
		  $results = jz_db_query($link, "SELECT main_art FROM jz_nodes WHERE path = '$path'");
		  jz_db_close($link);
		  
		  if ($results->data[0]['main_art']) {
		    // Now let's make create the resized art IF needed
		    $this->artpath = stripSlashes($results->data[0]['main_art']);
		    return parent::getMainArt($dimensions,$createBlank);
		  } else if ($this->isLeaf() === false) { 
		    // Now let's see if we can get art from the tags
		    $tracks = $this->getSubNodes("tracks");
		    if (count($tracks) > 0){
		      $meta = $jzSERVICES->getTagData($tracks[0]->getDataPath());
		      // Did we get it?
		      if ($meta['pic_data'] <> ""){
				if ($dimensions){
				  // Now lets check or create or image and return the resized one
				  return $jzSERVICES->resizeImage("ID3:". $tracks[0]->getDataPath(), $dimensions);
				} else {
				  return "ID3:". $tracks[0]->getDataPath();
				}
		      } else {
			  	return $jzSERVICES->createImage($this->getDataPath(). ".jpg", $dimensions, $this->getName());
			  }
		    }
		  }
		  // inheritance is sweet.
		  return parent::getMainArt($dimensions,$createBlank);
		}
		
		/**
		* Sets the node's main art
		* 
		* @author Ben Dodson
		* @version 6/7/04
		* @since 6/7/04
		*/
		function addMainArt($image) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			$image = addslashes($image);
			if (!$link = jz_db_connect())
            	die ("could not connect to database.");
                     		
			$path = addSlashes($this->getPath("String"));
			$results = jz_db_query($link, "UPDATE jz_nodes SET main_art = '$image' WHERE path = '$path'");
			jz_db_close($link);
		}



		/**
		* Returns the miscellaneous artwork attached to the node.
		* 
		* @author 
		* @version 
		* @since 
		*/
		function getRandomArt() {}
		
		
		/**
		* Adds misc. artwork to the node.
		* 
		* @author 
		* @version 
		* @since 
		*/
		function addRandomArt($image) {}



		/**
		* Returns a brief description for the node.
		* 
		* @author Ben Dodson
		* @version 5/21/04
		* @since 5/21/04
		*/
		function getShortDescription() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT descr FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	if ($results->data[0]['descr']) {
	                     	return stripSlashes($results->data[0]['descr']);
	                } else { return false; }
		}
		
		
		/**
		* Adds a brief description.
		* 
		* @author Ben Dodson 
		* @version 5/21/04 
		* @since 5/21/04
		*/		
		function addShortDescription($text) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			$text = addSlashes($text);
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "UPDATE jz_nodes SET descr = '$text'
                                                     WHERE path = '$path'");

			jz_db_close($link);
		}


		/**
		* Returns the description of the node.
		* 
		* @author Ben Dodson
		* @version 5/21/04
		* @since 5/21/04
		*/
		function getDescription() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT longdesc FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	if ($results->data[0]['longdesc']) {
	                     	return stripSlashes($results->data[0]['longdesc']);
	                } else { return false; }
		}
		
		
		/**
		* Adds a description.
		* 
		* @author Ben Dodson
		* @version 5/21/04
		* @since 5/21/04
		*/		
		function addDescription($text) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			$text = addSlashes($text);
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "UPDATE jz_nodes SET longdesc = '$text'
                                                     WHERE path = '$path'");
			jz_db_close($link);

		}


		/**
		* Gets the overall rating for the node.
		* 
		* @author Ben Dodson
		* @version 6/7/04
		* @since 6/7/04
		*/
		function getRating() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT rating_val FROM jz_nodes WHERE path = '$path'");
                     	jz_db_close($link);
                     	return $results->data[0]['rating_val'];
		}
		
		
		/**
		* Add a rating for the node.
		* 
		* @author Ben Dodson
		* @version 6/7/04
		* @since 6/7/04
		*/		
		function addRating($rating, $weight = false) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db, $rating_weight, $jzUSER;
			
			if ($weight === false) {
			  $weight = $jzUSER->getSetting('ratingweight');
			}

			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     	
			$addRating = $rating * $weight;
			$addWeight = $weight;

                     	$path = addSlashes($this->getPath("String"));

			$results = jz_db_query($link, "SELECT rating,rating_count FROM jz_nodes WHERE path = '$path'");
			if ($results->data[0]['rating_count'] == 0) {
			  $rval = $rating;
			} else {
			  $rval = estimateRating(($results->data[0]['rating'] + $addRating) / ($results->data[0]['rating_count'] + $addWeight));
			}

			$results = jz_db_query($link, "UPDATE jz_nodes SET rating=rating+$addRating,
                                                     rating_count=rating_count+$addWeight, rating_val=$rval WHERE path = '$path'");
		
			if ($rating_weight > 0 && $this->getLevel() > 0) {
				$path = $this->getPath();
				array_pop($path);
				$next = &new jzMediaNode($path);
				$next->addRating($rating, $weight * $rating_weight);
			}
			jz_db_close($link);
		}
		
		/**
		* Gets the number of people who have rated this element
		* 
		* @author Ben Dodson
		* @version 6/11/04
		* @since 6/11/04
		*/
		function getRatingCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT rating_count FROM jz_nodes WHERE path = '$path'");
                     	jz_db_close($link);
                     	return $results->data[0]['rating_count'];
		}


		/**
		* Returns the node's discussion
		* 
		* @author Ben Dodson
		* @version 6/7/04
		* @since 6/7/04
		*/
		function getDiscussion() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	
                     	$results = jz_db_query($link, "SELECT * FROM jz_discussions WHERE path LIKE '$path' ORDER BY my_id");
                     	
                     	$discussion = array();
                     	$i = 0;
                     	foreach ($results->data as $key => $data) {
                     		$discussion[$i]['user'] = stripslashes($data['user']);
                     		$discussion[$i]['comment'] = stripslashes($data['comment']);
				$discussion[$i]['id'] = $data['my_id'];
				$discussion[$i]['date'] = $data['date_added'];
                     		
                     		$i++;
                     	}
                     	jz_db_close($link);
                     	return ($discussion == array()) ? false : $discussion;
		}
		
		
		
		
		/**
		* Adds a blurb to the node's discussion
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/15/04
		* @since 5/15/04
		*/		
		function addDiscussion($text,$username) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$text = addSlashes($text);
                     	$username = addSlashes($username);
                     	
                     	jz_db_query($link, "INSERT INTO jz_discussions(path,user,comment,date_added)
                     	                  VALUES('$path','$username','$text',".time().")") || die(jz_db_error($link));
			jz_db_close($link);
		}
		
		
		/**
		* Returns the year of the element;
		* if it is a leaf, returns the info from getMeta[year]
		* else, returns the first matching year it finds.
		* Entry is '-' for no year.
		* 
		* @author Ben Dodson
		* @version 5/21/04
		* @since 5/21/04
		*/		
		function getYear() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	if ($this->isLeaf()) {
                     		$results = jz_db_query($link, "SELECT year FROM jz_tracks WHERE path = '$path'");
				jz_db_close($link);
	                     	return $results->data[0]['year'];
                     	}
                     	else { 
	                     	$results = jz_db_query($link, "SELECT year FROM jz_tracks WHERE path LIKE '${path}/%' AND year != '-' ORDER BY path LIMIT 1");
				jz_db_close($link);
	                     	if ($results->rows > 0) {
	                     		return $results->data[0]['year'];
	                     	} else { return "-"; }
                     	}
		}
		
		
		/**
		* Returns a string that points to the location
		* where this node's non-jinzora-specific data should be stored
		* (album art, text, etc.)
		* 
		* @author Ben Dodson
		* @version 9/18/04
		* @since 9/18/04
		*/
		function getDataPath() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db, $allow_filesystem_modify;
									
			if ($allow_filesystem_modify) {
				if (!$link = jz_db_connect())
                     			die ("could not connect to database.");
                     		
                     		$path = addSlashes($this->getPath("String"));
                     		$results = jz_db_query($link, "SELECT filepath FROM jz_nodes WHERE path = '$path'");
				jz_db_close($link);
                     		return stripslashes($results->data[0]['filepath']);
			}
			else {
				return $this->data_dir;
			}
		}


		/**
		* Returns a string that points to the location
		* where this node's non-jinzora-specific data should be stored
		* (album art, text, etc.)
		* 
		* @author Ben Dodson
		* @version 9/18/04
		* @since 9/18/04
		*/
		function getFilePath() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db, $allow_filesystem_modify;
									
			if (!$link = jz_db_connect())
                   		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT filepath FROM jz_nodes WHERE path = '$path'");
			//jz_db_close($link);
                     	return stripslashes($results->data[0]['filepath']);
		}
		
		/**
		* Marks this element as hidden.
		* 
		* @author Ben Dodson
		* @version 9/18/04
		* @since 9/18/04
		*/
		function hide() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
									

			if (!$link = jz_db_connect())
                   		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE nodes SET hidden='true' WHERE path = '$path'");
                     	
                     	if ($this->isLeaf()) {
                     		jz_db_query($link, "UPDATE jz_tracks SET hidden='true' WHERE path = '$path'");
                     	}	
			jz_db_close($link);
		}


		/**
		* Unhides the element
		* 
		* @author Ben Dodson
		* @version 9/18/04
		* @since 9/18/04
		*/
		function unhide() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
									
			if (!$link = jz_db_connect())
                   		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET hidden='false' WHERE path = '$path'");
                     	
                     	if ($this->isLeaf()) {
                     		jz_db_query($link, "UPDATE jz_tracks SET hidden='false' WHERE path = '$path'");
                     	}
			jz_db_close($link);
		}
		// end global_include: overrides.php
		
	}

	class jzRawMediaTrack extends jzMediaTrackClass {
		// Meta stuff
		var $title;
		/**
		* Constructor wrapper for jzMediaTrack.
		* 
		* @author Ben Dodson
		* @version 5/11/04
		* @since 5/11/04
		*/
		function jzMediaTrack($par = array(),$mode="path") {
			$this->title = false;
			$this->_constructor($par,$mode);
		}
		
		
		/**
		* Returns the track's name (not filename)
		* 
		* @author Ben Dodson
		* @version 5/21/04
		* @since 5/21/04
		*/	
		function getName() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!isNothing($this->title)) {
			  return $this->title;
			}
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT trackname FROM jz_tracks WHERE path LIKE '$path'");
                     	$name = stripSlashes($results->data[0]['trackname']);
			if (isNothing($name)) {
			  return $this->path[sizeof($this->path)-1];
			} else {
			  return $name;
			}
		}
		
		
		/**
		* Returns the track's complete, useable file path.
		* $target is one of: user|host|general
		* 
		* @author Ben Dodson
		* @version 11/12/04
		* @since 5/21/04
		*/		
		function getFileName($target = "user") {
			if ($this->playpath === false || $this->playpath == "") {
				if (!$link = jz_db_connect())
	                     		die ("could not connect to database.");
	                     	$path = addSlashes($this->getPath("String"));
				$results = jz_db_query($link, "SELECT filepath FROM jz_nodes WHERE path LIKE '$path'");
				jz_db_close($link);
	                     	$string = stripSlashes($results->data[0]['filepath']);
	                     	if ($string == "" || $string == false) {
	                     		// ERROR?
	                     		return false;
	                     	}
	                     	$this->playpath = $string;
				
	                }
                     	return parent::getFileName($target);
		}


		/**
		* Returns the track's metadata as an array with the following keys:
		*
		* title
		* bitrate
		* frequency
		* filename [excluding path]
		* size
		* year
		* comment
		* length
		* number
		* genre
		* artist
		* album
		* lyrics
		* type [extension]
		* 
		* These are taken mostly from the ID3.
		*
		* @author
		* @version
		* @since
		*/		
		function getMeta($mode = "cache") {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if ($mode == "cache" && $this->meta != array()) {
				return $this->meta;
			}
			if ($mode == "cache") {
				$meta = array();
				if (!$link = jz_db_connect())
					die ("could not connect to database.");
					
				$path = addSlashes($this->getPath("String"));
				$results = jz_db_query($link, "SELECT jz_tracks.*,jz_nodes.name,jz_nodes.descr FROM jz_tracks,jz_nodes 
				WHERE jz_nodes.path = '$path' AND jz_nodes.path = jz_tracks.path");
				
				$meta['title'] = stripslashes($results->data[0]['trackname']);
				$meta['bitrate'] = stripslashes($results->data[0]['bitrate']);
				$meta['frequency'] = stripslashes($results->data[0]['frequency']);
				$meta['filename'] = stripslashes($results->data[0]['name']);
				$meta['size'] = stripslashes($results->data[0]['filesize']);
				$meta['year'] = stripslashes($results->data[0]['year']);
				$meta['comment'] = stripslashes($results->data[0]['descr']);
				$meta['length'] = stripslashes($results->data[0]['length']);
				$meta['number'] = stripslashes($results->data[0]['number']);
				$meta['genre'] = stripslashes($results->data[0]['genre']);
				$meta['artist'] = stripslashes($results->data[0]['artist']);
				$meta['album'] = stripslashes($results->data[0]['album']);
				$meta['lyrics'] = stripslashes($results->data[0]['lyrics']);
				$meta['type'] = stripslashes($results->data[0]['extension']);
				
				if (isNothing($meta['type'])) {
				  $meta = parent::getMeta("file");
				  $this->setMeta($meta,"cache");
				}

				return $meta;
			}
			else {
				return parent::getMeta($mode);
			}
		}
		
		/**
		* Sets the track's meta information.
		* $meta is an array of meta fields to set.
		* $mode specifies where to update the meta,
		* false means do it in the cache and in the id3.
		* 
		* @author Ben Dodson
		* @version 10/13/04
		* @since 10/13/04
		*/		
		function setMeta($meta, $mode = false) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if ($mode == false) {
				$this->setMeta($meta,"file");
				$this->setMeta($meta,"cache");
			}
			if ($mode == "cache") {
				if (!$link = jz_db_connect())
                     			die ("could not connect to database.");
			
				$slashedFilePath = addSlashes($this->getPath("String"));
				$updatesql = "UPDATE jz_tracks SET valid = 'true'";
				
				
				if (isset($meta['title']))
					$updatesql .= ", trackname = '" . addSlashes($meta['title']) ."'";
	                     	if (isset($meta['bitrate']))
	                     		$updatesql .= ", bitrate = '" . addSlashes($meta['bitrate']) ."'";
	                     	if (isset($meta['frequency']))
	                     		$updatesql .= ", frequency = '" . addSlashes($meta['frequency']) . "'";
	                     	if (isset($meta['size']))
	                     		 $updatesql .= ", filesize = '" . addSlashes($meta['size']) . "'";
	                     	if (isset($meta['year']))
	                     		 $updatesql .= ", year = '" . addSlashes($meta['year']) . "'";
	                     	
	                     	if (isset($meta['length']))
	                     		$updatesql .= ", length = '" . addSlashes($meta['length']) . "'";
	                     	if (isset($meta['number']))
	                     		$updatesql .= ", number = '" . addSlashes($meta['number']) . "'";
	                     	if (isset($meta['genre']))
	                     		$updatesql .= ", genre = '" . addSlashes($meta['genre']) . "'";
	                     	if (isset($meta['artist']))
	                     		$updatesql .= ", artist = '" . addSlashes($meta['artist']) . "'";
	                     	if (isset($meta['album']))
	                     		$updatesql .= ", album = '" . addSlashes($meta['album']) . "'";
	                     	if (isset($meta['lyrics']))
	                     		$updatesql .= ", lyrics = '" . addSlashes($meta['lyrics']) . "'";
	                     	if (isset($meta['type']))
	                     		$updatesql .= ", extension = '" . addSlashes($meta['type']) . "'";
						
				$updatesql .= " WHERE path LIKE '$slashedFilePath'";
				jz_db_query($link,$updatesql);
				
				
				
				$updatesql = "UPDATE jz_nodes SET valid = 'true'";								
				if (isset($meta['filename']))
	                     		 $updatesql .= ", name = '" . addSlashes($meta['filename']) . "'";
	                    	if (isset($meta['comment']))
	                     		$updatesql .= ", descr = '" . addSlashes($meta['comment']) . "'";		
	                     		
				$updatesql .= " WHERE path LIKE '$slashedFilePath'";
				jz_db_query($link,$updatesql);
			}
			else {
				return parent::setMeta($meta,$mode);
			}
		}

		/**
		* Returns the track's lyrics.
		* 
		* @author Ben Dodson
		* @version 5/21/04
		* @since 5/21/04
		*/		
		function getLyrics() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT lyrics FROM jz_tracks WHERE path LIKE '$path'");
                     	return stripSlashes($results->data[0]['lyrics']);
		}
		
		//////////////////////////////////////
		// begin global_include: overrides.php
		/* * * * * * * * * * * * * * * * * * *
		 *            Overrides              *
		 * * * * * * * * * * * * * * * * * * */
		
		/**
		* Returns the date the node was added.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function getDateAdded() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT date_added FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	return $results->data[0]['date_added'];
		}

/**
 * Checks whether or not
 * this media has been updated within $days days.
 * If not, returns false. Otherwise, returns
 * the number of days since the most recent media was added.
 *
 * @author Ben Dodson
 * @version 4/9/05
 * @since 4/9/05
 **/
function newSince($days) {
  $time = $curtime = time();
  $time -= ($days*24*60*60);

  if ($this->getLevel() == 0) {
    $pathstring = "%";
  } else {
    $pathstring = $this->getPath("String");
    $pathstring .= "/%";
  }
  $sql = "SELECT date_added FROM jz_nodes WHERE ";
  $sql .= "path LIKE '$pathstring' AND date_added >= $time ORDER BY date_added desc LIMIT 1";


  if (!$link = jz_db_connect())
    die ("could not connect to database.");
  
  $results = jz_db_query($link, $sql);
  jz_db_close($link);

  if (sizeof($results->data) == 0) {
    return false;
  } else {
    return ceil(abs($curtime - $results->data[0]['date_added']) / (24*60*60));
  }
}


/**
 * Returns the element's ID
 *
 * @author Ben Dodson
 * @version 3/11/05
 * @since 3/11/05
 **/
function getID() {
  if (isset($this->myid) && $this->myid !== false) {
    return $this->myid;
  } else {
    if (!$link = jz_db_connect())
      die ("could not connect to database.");
    
    $path = addSlashes($this->getPath("String"));
    $results = jz_db_query($link, "SELECT my_id FROM jz_nodes WHERE path = '$path'");
    jz_db_close($link);
    return $results->data[0]['my_id'];
  }
}

/**
 * Converts an id to a path
 *
 * @author Ben Dodson
 * @version 3/11/05
 * @since 3/11/05
 **/
function idToPath($id) {
    if (!$link = jz_db_connect())
      die ("could not connect to database.");
    
    $results = jz_db_query($link, "SELECT path FROM jz_nodes WHERE my_id = '$id'");
    jz_db_close($link);
    return $results->data[0]['path'];
}

		/**
		* Returns the number of times the node has been played.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function getPlayCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT playcount FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	return $results->data[0]['playcount'];
		}
		
		
		/**
		* Increments the node's playcount, as well
		* as the playcount of its parents.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function increasePlayCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
			$sql = "UPDATE jz_nodes SET playcount = playcount+1, lastplayed = " . time();

                     	jz_db_query($link, "$sql  WHERE path = '$path'");
			                     	
                     	if (sizeof($ar = $this->getPath()) > 0) {
                     		array_pop($ar);
                     		$next = &new jzMediaNode($ar);
				$next->increasePlayCount();
                     	}
			jz_db_close($link);
		}
	
		/**
		* Sets the elements playcount.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function setPlayCount($n) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET playcount = $n WHERE path = '$path'");
			jz_db_close($link);
		}


	
               /**
		* Increments the node's view count
		*
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 3/15/2005
		* @since 3/15/2005
		*/
		function increaseViewCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET viewcount = viewcount+1 WHERE path = '$path'");
                     	
			jz_db_close($link);
		}


		/**
		* Returns the number of times the node has been viewed.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 3/15/2005
		* @since 3/15/2005
		*/
		function getViewCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT viewcount FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	return $results->data[0]['viewcount'];
		}


		/**
		* Returns the number of times the node has been downloaded.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function getDownloadCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT dlcount FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	return $results->data[0]['dlcount'];
		}
		
		
		/**
		* Increments the node's download count, as well
		* as the count of its parents.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function increaseDownloadCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET dlcount = dlcount+1 WHERE path = '$path'");
                     	
                     	if (sizeof($ar = $this->getPath()) > 0) {
                     		array_pop($ar);
                     		$next = &new jzMediaNode($ar);
				$next->increasePlayCount();
                     	}
			jz_db_close($link);
		}

		/**
		* Sets the elements download count.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/2004
		* @since 5/14/2004
		*/
		function setDownloadCount($n) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET dlcount = $n WHERE path = '$path'");
			jz_db_close($link);
		}

		/**
		* Returns the main art for the node.
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/14/04
		* @since 5/14/04
		*/
		function getMainArt($dimensions = false, $createBlank = true) {
		  global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db,$jzSERVICES;

		  if (!$link = jz_db_connect())
		    die ("could not connect to database.");
		  
		  $path = addSlashes($this->getPath("String"));
		  $results = jz_db_query($link, "SELECT main_art FROM jz_nodes WHERE path = '$path'");
		  jz_db_close($link);
		  
		  if ($results->data[0]['main_art']) {
		    // Now let's make create the resized art IF needed
		    $this->artpath = stripSlashes($results->data[0]['main_art']);
		    return parent::getMainArt($dimensions,$createBlank);
		  } else if ($this->isLeaf() === false) { 
		    // Now let's see if we can get art from the tags
		    $tracks = $this->getSubNodes("tracks");
		    if (count($tracks) > 0){
		      $meta = $jzSERVICES->getTagData($tracks[0]->getDataPath());
		      // Did we get it?
		      if ($meta['pic_name'] <> ""){
			if ($dimensions){
			  // Now lets check or create or image and return the resized one
			  return $jzSERVICES->resizeImage("ID3:". $tracks[0]->getDataPath(), $dimensions);
			} else {
			  return "ID3:". $tracks[0]->getDataPath();
			}
		      }
		    }
		  }
		  // inheritance is sweet.
		  return parent::getMainArt($dimensions,$createBlank);
		}
		
		/**
		* Sets the node's main art
		* 
		* @author Ben Dodson
		* @version 6/7/04
		* @since 6/7/04
		*/
		function addMainArt($image) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			$image = addslashes($image);
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "UPDATE jz_nodes SET main_art = '$image' WHERE path = '$path'");
			jz_db_close($link);
		}



		/**
		* Returns the miscellaneous artwork attached to the node.
		* 
		* @author 
		* @version 
		* @since 
		*/
		function getRandomArt() {}
		
		
		/**
		* Adds misc. artwork to the node.
		* 
		* @author 
		* @version 
		* @since 
		*/
		function addRandomArt($image) {}



		/**
		* Returns a brief description for the node.
		* 
		* @author Ben Dodson
		* @version 5/21/04
		* @since 5/21/04
		*/
		function getShortDescription() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT descr FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	if ($results->data[0]['descr']) {
	                     	return stripSlashes($results->data[0]['descr']);
	                } else { return false; }
		}
		
		
		/**
		* Adds a brief description.
		* 
		* @author Ben Dodson 
		* @version 5/21/04 
		* @since 5/21/04
		*/		
		function addShortDescription($text) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			$text = addSlashes($text);
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "UPDATE jz_nodes SET descr = '$text'
                                                     WHERE path = '$path'");

			jz_db_close($link);
		}


		/**
		* Returns the description of the node.
		* 
		* @author Ben Dodson
		* @version 5/21/04
		* @since 5/21/04
		*/
		function getDescription() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT longdesc FROM jz_nodes WHERE path = '$path'");
			jz_db_close($link);
                     	if ($results->data[0]['longdesc']) {
	                     	return stripSlashes($results->data[0]['longdesc']);
	                } else { return false; }
		}
		
		
		/**
		* Adds a description.
		* 
		* @author Ben Dodson
		* @version 5/21/04
		* @since 5/21/04
		*/		
		function addDescription($text) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			$text = addSlashes($text);
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "UPDATE jz_nodes SET longdesc = '$text'
                                                     WHERE path = '$path'");
			jz_db_close($link);

		}


		/**
		* Gets the overall rating for the node.
		* 
		* @author Ben Dodson
		* @version 6/7/04
		* @since 6/7/04
		*/
		function getRating() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT rating_val FROM jz_nodes WHERE path = '$path'");
                     	jz_db_close($link);
                     	return $results->data[0]['rating_val'];
		}
		
		
		/**
		* Add a rating for the node.
		* 
		* @author Ben Dodson
		* @version 6/7/04
		* @since 6/7/04
		*/		
		function addRating($rating, $weight = false) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db, $rating_weight, $jzUSER;
			
			if ($weight === false) {
			  $weight = $jzUSER->getSetting('ratingweight');
			}

			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     	
			$addRating = $rating * $weight;
			$addWeight = $weight;

                     	$path = addSlashes($this->getPath("String"));

			$results = jz_db_query($link, "SELECT rating,rating_count FROM jz_nodes WHERE path = '$path'");
			if ($results->data[0]['rating_count'] == 0) {
			  $rval = $rating;
			} else {
			  $rval = estimateRating(($results->data[0]['rating'] + $addRating) / ($results->data[0]['rating_count'] + $addWeight));
			}

			$results = jz_db_query($link, "UPDATE jz_nodes SET rating=rating+$addRating,
                                                     rating_count=rating_count+$addWeight, rating_val=$rval WHERE path = '$path'");
		
			if ($rating_weight > 0 && $this->getLevel() > 0) {
				$path = $this->getPath();
				array_pop($path);
				$next = &new jzMediaNode($path);
				$next->addRating($rating, $weight * $rating_weight);
			}
			jz_db_close($link);
		}
		
		/**
		* Gets the number of people who have rated this element
		* 
		* @author Ben Dodson
		* @version 6/11/04
		* @since 6/11/04
		*/
		function getRatingCount() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT rating_count FROM jz_nodes WHERE path = '$path'");
                     	jz_db_close($link);
                     	return $results->data[0]['rating_count'];
		}


		/**
		* Returns the node's discussion
		* 
		* @author Ben Dodson
		* @version 6/7/04
		* @since 6/7/04
		*/
		function getDiscussion() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	
                     	$results = jz_db_query($link, "SELECT * FROM jz_discussions WHERE path LIKE '$path' ORDER BY my_id");
                     	
                     	$discussion = array();
                     	$i = 0;
                     	foreach ($results->data as $key => $data) {
                     		$discussion[$i]['user'] = stripslashes($data['user']);
                     		$discussion[$i]['comment'] = stripslashes($data['comment']);
				$discussion[$i]['id'] = $data['my_id'];
				$discussion[$i]['date'] = $data['date_added'];
                     		
                     		$i++;
                     	}
                     	jz_db_close($link);
                     	return ($discussion == array()) ? false : $discussion;
		}
		
		
		
		
		/**
		* Adds a blurb to the node's discussion
		* 
		* @author Ben Dodson <bdodson@seas.upenn.edu>
		* @version 5/15/04
		* @since 5/15/04
		*/		
		function addDiscussion($text,$username) {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$text = addSlashes($text);
                     	$username = addSlashes($username);
                     	
                     	jz_db_query($link, "INSERT INTO jz_discussions(path,user,comment,date_added)
                     	                  VALUES('$path','$username','$text',".time().")") || die(jz_db_error($link));
			jz_db_close($link);
		}
		
		
		/**
		* Returns the year of the element;
		* if it is a leaf, returns the info from getMeta[year]
		* else, returns the first matching year it finds.
		* Entry is '-' for no year.
		* 
		* @author Ben Dodson
		* @version 5/21/04
		* @since 5/21/04
		*/		
		function getYear() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
			
			if (!$link = jz_db_connect())
                     		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	if ($this->isLeaf()) {
                     		$results = jz_db_query($link, "SELECT year FROM jz_tracks WHERE path = '$path'");
				jz_db_close($link);
	                     	return $results->data[0]['year'];
                     	}
                     	else { 
	                     	$results = jz_db_query($link, "SELECT year FROM jz_tracks WHERE path LIKE '${path}/%' AND year != '-' ORDER BY path LIMIT 1");
				jz_db_close($link);
	                     	if ($results->rows > 0) {
	                     		return $results->data[0]['year'];
	                     	} else { return "-"; }
                     	}
		}
		
		
		/**
		* Returns a string that points to the location
		* where this node's non-jinzora-specific data should be stored
		* (album art, text, etc.)
		* 
		* @author Ben Dodson
		* @version 9/18/04
		* @since 9/18/04
		*/
		function getDataPath() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db, $allow_filesystem_modify;
									
			if ($allow_filesystem_modify) {
				if (!$link = jz_db_connect())
                     			die ("could not connect to database.");
                     		
                     		$path = addSlashes($this->getPath("String"));
                     		$results = jz_db_query($link, "SELECT filepath FROM jz_nodes WHERE path = '$path'");
				jz_db_close($link);
                     		return stripslashes($results->data[0]['filepath']);
			}
			else {
				return $this->data_dir;
			}
		}


		/**
		* Returns a string that points to the location
		* where this node's non-jinzora-specific data should be stored
		* (album art, text, etc.)
		* 
		* @author Ben Dodson
		* @version 9/18/04
		* @since 9/18/04
		*/
		function getFilePath() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db, $allow_filesystem_modify;
									
			if (!$link = jz_db_connect())
                   		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	$results = jz_db_query($link, "SELECT filepath FROM jz_nodes WHERE path = '$path'");
			//jz_db_close($link);
                     	return stripslashes($results->data[0]['filepath']);
		}
		
		/**
		* Marks this element as hidden.
		* 
		* @author Ben Dodson
		* @version 9/18/04
		* @since 9/18/04
		*/
		function hide() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
									

			if (!$link = jz_db_connect())
                   		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE nodes SET hidden='true' WHERE path = '$path'");
                     	
                     	if ($this->isLeaf()) {
                     		jz_db_query($link, "UPDATE jz_tracks SET hidden='true' WHERE path = '$path'");
                     	}	
			jz_db_close($link);
		}


		/**
		* Unhides the element
		* 
		* @author Ben Dodson
		* @version 9/18/04
		* @since 9/18/04
		*/
		function unhide() {
			global $sql_type,$sql_pw,$sql_usr,$sql_socket,$sql_db;
									
			if (!$link = jz_db_connect())
                   		die ("could not connect to database.");
                     		
                     	$path = addSlashes($this->getPath("String"));
                     	jz_db_query($link, "UPDATE jz_nodes SET hidden='false' WHERE path = '$path'");
                     	
                     	if ($this->isLeaf()) {
                     		jz_db_query($link, "UPDATE jz_tracks SET hidden='false' WHERE path = '$path'");
                     	}
			jz_db_close($link);
		}
		// end global_include: overrides.php
	}
?>
