<?php
	/**
	* - JINZORA | Web-based Media Streamer -  
	* 
	* Jinzora is a Web-based media streamer, primarily desgined to stream MP3s 
	* (but can be used for any media file that can stream from HTTP). 
	* Jinzora can be integrated into a CMS site, run as a standalone application, 
	* or integrated into any PHP website.  It is released under the GNU GPL.
	* 
	* - Resources -
	* - Jinzora Author: Ross Carlson <ross@jasbone.com>
	* - Web: http://www.jinzora.org
	* - Documentation: http://www.jinzora.org/docs	
	* - Support: http://www.jinzora.org/forum
	* - Downloads: http://www.jinzora.org/downloads
	* - License: GNU GPL <http://www.gnu.org/copyleft/gpl.html>
	* 
	* - Contributors -
	* Please see http://www.jinzora.org/team.html
	* 
	* - Code Purpose -
	* Contains the basic Jukebox class
	*
	* @since 2/9/05
	* @author Ross Carlson <ross@jinzora.org>
	*/
	
	class jzJukebox {
	
		/**
		* Constructor for the class.
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		*/
		function jzJukebox(){
			global $include_path, $jbArr;
			
			// Now let's include the settings for jukebox
			include_once($include_path. "jukebox/settings.php");
			
			// Ok, now we need to include the right subclass for this player
			if (!isset($_SESSION['jb_id'])){ $_SESSION['jb_id'] = 0; }
			
			// Now let's make sure they have installed the jukebox
			if (!isset($jbArr[$_SESSION['jb_id']]['type'])){
				echo "It looks like you have not yet setup a jukebox to be used with Jinzora&trade;<br><br>";
				echo "You'll need to edit the file settings.php in the 'jukebox' directory to setup your jukebox before you can proceed";
				exit();
			}
			
			include_once($include_path. "jukebox/jukeboxes/". $jbArr[$_SESSION['jb_id']]['type']. ".php");
		}
		
		/**
		* Returns a keyed array showing all the functions that this jukebox supports
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param return Returns a keyed array of the jukeboxe's abilities
		*/
		function jbAbilities(){
			return returnJBAbilities();
		}
		
		/**
		* Returns the stats of the jukebox
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param return Returns a keyed array of the jukeboxe's abilities
		*/
		function returnJBStats(){
			return retJBStats();
		}
		
		/**
		* Connects to the player and returns true or false based on what happens
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param return Returns true or false
		*/
		function connect(){
			return playerConnect();
		}
		
		/**
		* Displays add on tools next to the playlist box
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param return Returns the path to the current track
		*/
		function getAddOnTools(){
			return getAllAddOnTools();
		}
		
		/**
		* returns the currently playing track path so we can get the node
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param return Returns the path to the current track
		*/
		function getCurrentTrackPath(){
			return getCurTrackPath();
		}
		
		/**
		* returns the currently playing track number
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param return Returns the currently playling track number
		*/
		function getCurrentPlayingTrack(){
			return getCurPlayingTrack();
		}
		
		/**
		* returns the currently playing playlist
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param return Returns the currently playling playlist
		*/
		function getCurrentPlaylist(){
			return getCurPlaylist();
		}
		
		/**
		* Passes a playlist to the jukebox
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param $playlist The playlist that we are passing
		*/
		function passPlaylist($playlist){
			playlist($playlist);
		}
		
		/**
		* Passes a command to the jukebox player
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param $command The command that we passed to the player
		*/
		function passCommand($command){
			control($command);
		}
		
		/**
		* Returns the status of the player
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		*/
		function getPlayerStatus(){
			return getStatus();
		}
		
		/**
		* Gets the current track that is playing by the jukebox
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		*/
		function getCurrentTrackName(){
			return getCurTrackName();
		}
		
		/**
		* Gets the ammount of time remaining in the current track
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param return returns the amount of time remaining in seconds
		*/
		function getCurrentTrackRemaining(){
			return getCurTrackRemaining();
		}		
		
		/**
		* Gets the length of the current track
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param return returns the amount of time remaining in seconds
		*/
		function getCurrentTrackLength(){
			return getCurTrackLength();
		}
		
		/**
		* Gets the length location of the track (how many seconds into the track we are)
		* 
		* @author Ross Carlson
		* @version 2/9/05
		* @since 2/9/05
		* @param return returns the amount of time played in seconds
		*/
		function getCurrentTrackLocation(){
			return getCurTrackLocation();
		}
	}
?>